/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:	ResourceExtendSet.c
 *
 * Date:	May 1994
 *
 * Purpose:  	The extension of the Athena Widget Set including
 *              communication between application and resources
 *
 * Author:	P. Peters
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
 

#include <ResourceExtendSet.h>

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Variable: current_active_widget
 * Argument: none
 * resource: none
 * purpose : declaration of global variable used by xRESPrintString
 *           (& xRESPrintStringnPause)
 * remarks : VERY IMPORTANT: use the xRESInitVariables() every beginning of self-written
 *           function see the structure of xInitManager.c and Agent.
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
Widget current_active_widget;

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESInitVariables
 * Argument:  Widget current_widget
 * resource: none
 * purpose : initialize the variable for the output
 * remarks : NECESSARY for every declared function, which use xRES-functions
 *	     Must be put in the first line after the variable declaring
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
int xRESInitVariables( Widget current_widget )
{
  current_active_widget = current_widget;  
  return( Ok_Status );
}

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESGetnString
 * Argument: string_number(in) , head_string(in) , return_string(out)
 * resource: none
 * purpose : Gets the n-e substring out of the head-string
 * remarks : returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
int xRESGetnString( string_number , head_string, return_string )
  int string_number;
  char *head_string , *return_string;
{
  int current_string , found_string , status;
  char *subchar , *substring;

  substring      = return_string;
  subchar        = head_string;
  current_string = 0;
  found_string   = 0;
  while( ( *subchar != '\0' ) && ( found_string != 1 ) )
  {
    *substring++ = *subchar++;
    if( *subchar == ' ' || *subchar == '\0' )
    {
      *substring = '\0';
      if( current_string == string_number )
        found_string = 1;
      else
      {
        if( *subchar != '\0' )
        {
          substring = return_string;
          current_string++;
          *subchar++;
        }
      }
    }
  }
  if( current_string != string_number )
  {
    *return_string = '\0';
    status = Fault_Status;
  }
  else
    status = Ok_Status;
  return( status );
}

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESPrintString
 * Argument: output_widget(in) , output_string(in)
 * resource: none
 * purpose : prints an outputtext to the outputwidget with WcSetValue
 * remarks : use the string name of the output widget, only for AsciiSrc.
 *  	     returns the status.
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
int xRESPrintString( output_widget , output_string )
  char *output_widget;
  char *output_string;
{
  char *buf;
  static char* fmt = "%s.string(String):%s";
  Widget status_widget;
  int return_status = Fault_Status;

  buf = XtMalloc( WcStrLen( fmt ) + WcStrLen( output_widget ) + WcStrLen( output_string ) + 1 );
  sprintf( buf , fmt , output_widget , output_string );
  if( ( status_widget = WcFullNameToWidget( current_active_widget , output_widget ) ) != NULL )
  {
    WcSetValue( current_active_widget , buf );
    XFlush( XtDisplay( status_widget ) );
    return_status = Ok_Status;
  }
  else
    printf( "Warning : %s\n" , pWarningMessage1 );
  XtFree( buf );
  return(  return_status );
}

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESPrintStringnPause
 * Argument: output_widget(in) , output_string(in) , time_sec(in)
 * resource: none
 * purpose : Prints the output_string with xRESPrintString and pauses time_sec seconds
 * remarks : returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
int xRESPrintStringnPause( output_widget , output_string , time_sec )
  char *output_widget , *output_string;
  int time_sec;
{
  int return_status;

  return_status = xRESPrintString( output_widget , output_string );
  if( return_status == Ok_Status )
  { 
    sleep( time_sec );
  }
  return( return_status );
}

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESCheckIntegerString
 * Argument: int_string(in)
 * resource: none
 * purpose : Check if the string contains only digits
 * remarks : returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
int xRESCheckIntegerString( char *int_string )
{
  int ret_stat = Ok_Status;

  while( *int_string != '\0' )
  {
    if( !isdigit( *int_string ) ) ret_stat = Fault_Status;
    int_string++;
  }
  return( ret_stat );
}

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESCheckIntegerInRange
 * Argument: number(in), low_range(in) , high_range(in)
 * resource: none
 * purpose : check if an integer is in the specified range
 * remarks : returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
int xRESCheckIntegerInRange( int number, int low_range, int high_range)
{
  int ret_stat = Ok_Status;

  if( number < low_range ) ret_stat = Fault_Status;
  if( number > high_range ) ret_stat = Fault_Status;
  return( ret_stat );
}
