/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:	snmpMINManFile.c
 *
 * Date:	May 1994
 *
 * Purpose:	Initialize partymib configuration files.
 *
 * Author:	Michel van Ooijen & Edward Veluwenkamp
 *		Peter Peters
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
 
/* -----------------------------------------------------------------------
 *  include files
 * -----------------------------------------------------------------------
 */
#include <snmpInitManager.h>
#include <snmpInitManFile.h>
#include <ResourceExtendSet.h>
#include <ErrorWarningMessages.h>
#include <xInitManager.h>
/* ------------------------------------------------------------------------------------------
 *  Function : MINopenmanager 
 *  Arguments: fp, a filepointer (IN/OUT)
 *             priv, pointer of an array priveledge (IN) 
 *             managername, pointer of an array managername (IN)
 *             typefile, pointer of an array filename (IN)
 *             inifile, pointer of an array init filename (IN)
 *  Purpose  : This function are opening the manager cmu files
 *  Remarks  : return -1, if a file not can open/write or append 
 * ------------------------------------------------------------------------------------------
 */
int MINopenmanager(fp,priv,managername,typefile,inifile,output_widget_name)
  FILE **fp;
  char *priv;
  char *managername;
  char *typefile;
  char *inifile;
  char *output_widget_name;
{
  char filename[FILELEN] = "";
  char xIMtext[ maximum_string_length ];

  xIMtext[0] = '\0';
  strcpy( xIMtext , "Opening \0" );
  if( (char *) getenv( "SNMP_PARTYMIB_PATH" ) != NULL )
	strcpy( filename, (const char *) getenv( "SNMP_PARTYMIB_PATH" ) );
  strcat(filename,managername);
  strcat(filename,typefile);
  if ((*fp=fopen(filename,priv)) == (FILE *) NULL) /* opening and testing */
   {
   if ((*fp=fopen(filename,"w")) == (FILE *) NULL)
     {
     strcpy( xIMtext , "ERROR: Unable to open " );
     strcat( xIMtext , filename );
     strcat( xIMtext , " for saving.\0" );
     xRESPrintStringnPause( output_widget_name , xIMtext , 4 );
     return(-1);
     }
   else
     {
      strcpy( xIMtext , "Opening " );
      strcat( xIMtext , filename );
      strcat( xIMtext , ".\0" );
      xRESPrintStringnPause( output_widget_name , xIMtext , 2 );
     MINemptyfile(*fp,inifile/*managername*/,output_widget_name);
     fclose(*fp);
     if ((*fp=fopen(filename,priv)) == (FILE *) NULL)
       return(-1);
     else 
       { 
       return(0);
       }
     }
   }
 else
   {
   if (priv[0] == 'r')
     strcpy( xIMtext , "Opening " );
     strcat( xIMtext , filename );
     strcat( xIMtext , ".\0" );
     xRESPrintStringnPause( output_widget_name , xIMtext , 1 );
   }
 if (priv[0] == 'w')
   MINemptyfile(*fp,inifile,output_widget_name);
 return(0);
 }
 
 
/* ------------------------------------------------------------------------------------------
 *  Function : MINopenmanagerfiles 
 *  Arguments: fp, a filepointer array of filepointers (8 pointers) (IN/OUT)
 *             priv, pointer of an array priveledge (IN) 
 *             managername, pointer of an array managername (IN)
 *  Purpose  : Opening party, context, view and acl cmu file.
 *  Remarks  : return -1, if a file not can open/write or append
 * ------------------------------------------------------------------------------------------
*/
int MINopenmanagerfiles(fp,priv,managername,output_widget_name)
 FILE **fp;
 char *priv;
 char *managername;
 char *output_widget_name;
 {
 /* opening four manager files */
 if (MINopenmanager(&fp[0],priv,managername,PARTYFILE,PARTYINIFILE,output_widget_name) == -1)
   return(-1);
 if (MINopenmanager(&fp[1],priv,managername,CONTEXTFILE,CONTEXTINIFILE,output_widget_name) == -1)
   return(-1);
 if (MINopenmanager(&fp[2],priv,managername,VIEWFILE,VIEWINIFILE,output_widget_name) == -1)
   return(-1);
 if (MINopenmanager(&fp[3],priv,managername,ACLFILE,ACLINIFILE,output_widget_name) == -1)
   return(-1);
 return 0;
 }


/* ------------------------------------------------------------------------------------------
 *  Function : MINopenagent  
 *  Arguments: fp, a filepointer (IN/OUT)
 *             agentname, pointer of an array agentname (IN)
 *             typefile, pointer of an array filename (IN)
 *  Purpose  : Opening an agent file 
 *  Remarks  : return -1, if a file not can open/write or append
 * ------------------------------------------------------------------------------------------
*/
int MINopenagent(fp,agentname,typename,output_widget_name)
  FILE **fp;
  char *agentname;
  char *typename;
  char *output_widget_name;
  {
  char filename[FILELEN] = "";
  char xIMtext[ maximum_string_length ];

  if( (char *) getenv( "SNMP_PARTYMIB_PATH" ) != NULL )
	strcpy( filename, (const char *) getenv( "SNMP_PARTYMIB_PATH" ) );
  strcat(filename,agentname);
  strcat(filename,typename);
  if ((*fp=fopen(filename,"r")) == (FILE *) NULL)
    {
    strcpy( xIMtext , "ERROR: Unable to open " );
    strcat( xIMtext , filename );
    strcat( xIMtext , " for reading.\0" );
    xRESPrintStringnPause( output_widget_name , xIMtext , 4 );
    exit(-1);
    } else {
    strcpy( xIMtext , "Opening " );
    strcat( xIMtext , filename );
     strcat( xIMtext , ".\0" );
    xRESPrintStringnPause( output_widget_name , xIMtext , 1 );
    }
  }
 
 
/* ------------------------------------------------------------------------------------------
 *  Function : MINopenagentfiles
 *  Arguments: fp, a filepointer array of filepointers (8 pointers) (IN/OUT)
 *             agentname, pointer of an array agentname (IN)
 *  Purpose  : Opening the party, context, view and acl cmu files  
 *  Remarks  : return -1, if a file not can open/write or append
 * ------------------------------------------------------------------------------------------
*/
int MINopenagentfiles(fp,agentname,output_widget_name)
 FILE **fp;
 char *agentname;
 char *output_widget_name;
 {
 /* opening four agent files */
 if (MINopenagent(&fp[4],agentname,PARTYFILE,output_widget_name) == -1)
   return(-1);
 if (MINopenagent(&fp[5],agentname,CONTEXTFILE,output_widget_name) == -1)
   return(-1);
 if (MINopenagent(&fp[6],agentname,VIEWFILE,output_widget_name) == -1)
   return(-1);
 if (MINopenagent(&fp[7],agentname,ACLFILE,output_widget_name) == -1)
   return(-1);
 }
 


/* ------------------------------------------------------------------------------------------
 *  Function  : MINreadln
 *  Arguments : fp, a filepointer (IN/OUT)
 *              line, a array of chars (OUT) 
 *  Purpose   : read a line from the file (pointer)   
 *  Remarks   : return -1 if status getc() == EOF 
 * ------------------------------------------------------------------------------------------
*/
int MINreadln(fp, line)
 FILE *fp;
 char *line;
 {
 int i=0; /* local counter */
 int stop = 0; /* flag */
 int lineLen = 0;
 while ((stop == 0) && ((line[i]=getc(fp)) != EOF))
   {
   if (line[i] == '\n')
     stop = 1;
   i++;
   }
 if (feof(fp))  /* End of file */
   lineLen = -1;
 else lineLen = i-1;
 return(lineLen);
 }


/* ------------------------------------------------------------------------------------------
 *  Function  : MINgetWord 
 *  Arguments : word, a array of chars (OUT)
 *              line, a array of chars (IN/OUT)
 *              position, line position (IN/OUT)
 *              lineLen, line length (IN)
 *              state, integer status (IN/OUT)
 *              size, integer struct length (IN) 
 *  Purpose   : Search the first word from the current position in the line. 
 *  Remarks   : state 0 if a line begins with a '#' or '\n' and if the line position == lineLen 
 *              state 1 if a word is read
 * ------------------------------------------------------------------------------------------
*/
void MINgetWord(word,line,position,lineLen,state,size)
  char *word;
  char *line;
  int  *position;
  int  lineLen;
  int  *state;
  int  size;
  {
  int i;

  if ((line[0] == '#') || (line[0] == '\n')) /* comment line */
    (*state) = 0;
  else
    {

    while (isspace(line[*position]) && (*position < lineLen)) (*position)++; /* skip space */

    i = 0;
    while (!isspace(line[*position]) && (*position < lineLen) && (i < size)) /* store word */
      { 
      word[i++] = line[(*position)++]; 
      }
    word[i] = '\0'; /* string end */
    
    if (lineLen < (*position)) /* overflow */
      (*state) = 0;
    else; /* correct search */
    }
  }


/* ------------------------------------------------------------------------------------------
 *  Function  : MINfillparty 
 *  Arguments : partyuser, a pointer to the partyentrys (IN/OUT)
 *              line, a read line array of chars (IN)
 *              state, status integer (IN/OUT)
 *              recnr, record integer counter (IN/OUT)
 *              lineLen, integer (IN)
 *  Purpose   : This function fills the partyuser record. 
 *  Remarks   : 
 * ------------------------------------------------------------------------------------------
*/
void MINfillparty(partyuser,line,state,recnr,lineLen)
  PartyEntry partyuser;
  char       *line;
  int        *state;
  int        *recnr;
  int        lineLen;
  {
  char authPublicLengthstr[AUTHLEN];
  char privPublicLengthstr[PRIVLEN];
  int position;
  position = 0;
  switch(*state)
   {
   case IDENTITY :MINgetWord(partyuser[*recnr].FriendbyName,line,&position,lineLen,state,sizeof(partyuser[*recnr].FriendbyName));
                  MINgetWord(partyuser[*recnr].partyIdentity,line,&position,lineLen,state,sizeof(partyuser[*recnr].partyIdentity));
                  (*state)++;
                  break;
   case TRANSPORT:MINgetWord(partyuser[*recnr].partyTDomain,line,&position,lineLen,state,sizeof(partyuser[*recnr].partyTDomain));
                  MINgetWord(partyuser[*recnr].IPaddress,line, &position,lineLen, state,sizeof(partyuser[*recnr].IPaddress));
                  MINgetWord(partyuser[*recnr].partyTAddress,line,&position,lineLen,state,sizeof(partyuser[*recnr].partyTAddress));
                  (*state)++;
                  break;
   case PROTOCOL: MINgetWord(partyuser[*recnr].partyAuthProtocol,line,&position,lineLen, state,sizeof(partyuser[*recnr].partyAuthProtocol));
                  MINgetWord(partyuser[*recnr].partyPrivProtocol,line,&position,lineLen, state,sizeof(partyuser[*recnr].partyPrivProtocol));
                  (*state)++;
                  break;
   case LIFETIME: MINgetWord(partyuser[*recnr].partyAuthLifeTime,line,&position,lineLen, state,sizeof(partyuser[*recnr].partyAuthLifeTime));
                  MINgetWord(partyuser[*recnr].partyMaxMessageSize,line,&position,lineLen,state,sizeof(partyuser[*recnr].partyMaxMessageSize));
                  (*state)++;
                  break;
   case CLOCK   : MINgetWord(partyuser[*recnr].partyAuthClock,line,&position,lineLen,state,sizeof(partyuser[*recnr].partyAuthClock));
                  (*state)++;
                  break;
   case AUTH:     MINgetWord(partyuser[*recnr].partyAuthPrivate,line,&position,lineLen,state,sizeof(partyuser[*recnr].partyAuthPrivate));
                  MINgetWord(authPublicLengthstr,line, &position, lineLen, state,sizeof(authPublicLengthstr));
                  MINgetWord(partyuser[*recnr].partyAuthPublic,line,&position,lineLen,state,sizeof(partyuser[*recnr].partyAuthPublic));
                  (*state)++;
                  break;
   case  PRIV   : MINgetWord(partyuser[*recnr].partyPrivPrivate,line, &position, lineLen, state,sizeof(partyuser[*recnr].partyPrivPrivate));
                  MINgetWord(privPublicLengthstr,line, &position, lineLen, state,sizeof(privPublicLengthstr));
                  MINgetWord(partyuser[*recnr].partyPrivPublic,line, &position, lineLen, state,sizeof(partyuser[*recnr].partyPrivPublic));
                  if (*state != 0)
                    {
                    /*  MINprintpartyentry(partyuser,*recnr);*/
                    (*recnr)++;
                    }
    default     : *state = 1;
                  break;
    }
  }




/* ------------------------------------------------------------------------------------------
 *  Function  : MINfillcontext 
 *  Arguments : contextuser, a pointer to the contextentries (IN/OUT)
 *              line, a read line array of chars (IN)
 *              state, status integer (IN/OUT)
 *              recnr, record integer counter (IN/OUT)
 *              lineLen, integer (IN)
 *  Purpose   : This function fills the contextuser record. 
 *  Remarks   : 
 * ------------------------------------------------------------------------------------------
*/
void MINfillcontext(contextuser,line,state,recnr,lineLen)
  ContextEntry contextuser;
  char *line;
  int *state;
  int *recnr;
  int lineLen;
  {
  int position;
  char contextViewIndex[ARLEN];
  char contextIndex[ARLEN];
  position = 0;
  switch(*state)
    {
    case IDENTITY:MINgetWord(contextuser[*recnr].contextName,line,&position,lineLen, state,sizeof(contextuser[*recnr].contextName));
                  MINgetWord(contextuser[*recnr].contextIdentity,line,&position,lineLen,state,sizeof(contextuser[*recnr].contextIdentity));
                  (*state)++;
                  break;

    case TRANSPORT:MINgetWord(contextViewIndex,line, &position, lineLen, state,sizeof(contextViewIndex));
                   contextuser[*recnr].contextViewIndex = atoi(contextViewIndex);
                   MINgetWord(contextuser[*recnr].contextLocalEntity,line,&position,lineLen,state,sizeof(contextuser[*recnr].contextLocalEntity));
                   MINgetWord(contextuser[*recnr].contextLocalTime,line,&position,lineLen,state,sizeof(contextuser[*recnr].contextLocalTime));
                   (*state)++;
                   break;

     case PROTOCOL:MINgetWord(contextuser[*recnr].contextProxyDstParty,line, &position, lineLen, state,sizeof(contextuser[*recnr].contextProxyDstParty));
                   MINgetWord(contextuser[*recnr].contextProxySrcParty,line, &position, lineLen, state,sizeof(contextuser[*recnr].contextProxySrcParty));
                   MINgetWord(contextuser[*recnr].contextProxyContext,line, &position, lineLen, state,sizeof(contextuser[*recnr].contextProxyContext));
                   if (*state != 0)
                     {
                     /*  MINprintcontextentry(contextuser,*recnr);*/
                     (*recnr)++;
                     }
      default      : *state = 1;
                    break;
      }
  }



/* ------------------------------------------------------------------------------------------
 *  Function  : MINfillview 
 *  Arguments : viewuser, a pointer to the viewentries (IN/OUT)
 *              line, a read line array of chars (IN)
 *              state, status integer (IN/OUT)
 *              recnr, record integer counter (IN/OUT)
 *              lineLen, integer (IN)
 *  Purpose   : This function fills the viewuser record. 
 *  Remarks   : 
 * ------------------------------------------------------------------------------------------
*/
void MINfillview(viewuser,line,state,recnr,lineLen)
  ViewEntry viewuser;
  char *line;
  int *state;
  int *recnr;
  int lineLen;
  {
    int position;
    char viewIndex[ARLEN];
    position = 0;
    switch(*state)
      {
      case IDENTITY: MINgetWord(viewIndex,line, &position, lineLen, state,sizeof(viewIndex));
                     viewuser[*recnr].viewIndex = atoi(viewIndex);
                     MINgetWord(viewuser[*recnr].viewSubtree,line, &position, lineLen, state,sizeof(viewuser[*recnr].viewSubtree));
                     MINgetWord(viewuser[*recnr].viewMask,line, &position, lineLen, state,sizeof(viewuser[*recnr].viewMask));
                     MINgetWord(viewuser[*recnr].viewType,line, &position, lineLen, state,sizeof(viewuser[*recnr].viewType));
                     if (*state != 0)
                       {
		       /*printf("%d ",viewuser[*recnr].viewIndex);*/
                       /*MINprintviewentry(viewuser,*recnr); */
                       (*recnr)++;
                       }
                     (*state)++;
      default       : *state = 1;
                      break;
      }
  }



/* ------------------------------------------------------------------------------------------
 *  Function  : MINfillacl 
 *  Arguments : acluser, a pointer to the aclentries (IN/OUT)
 *              line, a read line array of chars (IN)
 *              state, status integer (IN/OUT)
 *              recnr, record integer counter (IN/OUT)
 *              lineLen, integer (IN)
 *  Purpose   : This function fills the acluser record. 
 *  Remarks   : 
 * ------------------------------------------------------------------------------------------
*/
void MINfillacl(acluser,line,state,recnr,lineLen)
  AclEntry acluser;
  char *line;
  int *state;
  int *recnr;
  int lineLen;
  {
    int position;
    position = 0;
    switch(*state)
      {
      case IDENTITY: MINgetWord(acluser[*recnr].aclTarget,line, &position, lineLen, state,sizeof(acluser[*recnr].aclTarget));
                     MINgetWord(acluser[*recnr].aclSubject,line, &position, lineLen, state,sizeof(acluser[*recnr].aclSubject));
                     MINgetWord(acluser[*recnr].aclResources,line, &position, lineLen, state,sizeof(acluser[*recnr].aclResources));
                     MINgetWord(acluser[*recnr].aclPrivileges,line, &position, lineLen, state,sizeof(acluser[*recnr].aclPrivileges));
                     if (*state != 0)
                       {
                       /*   MINprintaclentry(acluser,*recnr); */
                       (*recnr)++;
                        }
      default      : *state = 1;
                     break;
      }
  }



/* ------------------------------------------------------------------------------------------
 *  Function  : MINclearall
 *  Arguments : partyuser, a pointer to the partyentries (IN/OUT)
 *            : contextuser, a pointer to the contextentries (IN/OUT)
 *            : viewuser, a pointer to the viewentries (IN/OUT)
 *            : acluser, a pointer to the aclentries (IN/OUT)
 *  Purpose   : This function clears all the party, context, view and acl records. 
 *  Remarks   : 
 * ------------------------------------------------------------------------------------------
*/
void MINclearall(partyuser,contextuser,viewuser,acluser)
  PartyEntry partyuser;
  ContextEntry contextuser;
  ViewEntry viewuser;
  AclEntry acluser;
  {
  int i = 0;
  for (i = 0; i < lengthEntry; i++)
    {
    partyuser[i].partyIdentity[0] = '\0';
    contextuser[i].contextIdentity[0] = '\0';
    viewuser[i].viewIndex = -1;
    acluser[i].aclTarget[0] = '\0';
    }
  }


/* ------------------------------------------------------------------------------------------
 *  Function  : MINreadfiles
 *  Arguments : fp, a filepointer array of filepointers (8 pointers) (IN/OUT)
 *            : partyagent, a pointer to the partyentries (IN/OUT)
 *            : contextagent, a pointer to the contextentries (IN/OUT)
 *            : viewagent, a pointer to the viewentries (IN/OUT)
 *            : aclagent, a pointer to the aclentries (IN/OUT)
 *            : partymanager, a pointer to the partyentries (IN/OUT)
 *            : contextmanager, a pointer to the contextentries (IN/OUT)
 *            : viewmanager, a pointer to the viewentries (IN/OUT)
 *            : aclmanager, a pointer to the aclentries (IN/OUT)
 *  Purpose   : This function are reading the agent and manager files and fills the party, 
 *            : context, view and the acl records from the agent and the manager.
 *  Remarks   : 
 * ------------------------------------------------------------------------------------------
*/
void MINreadfiles(fp,partyagent,contextagent,viewagent,aclagent,
                  partyman,contextman,viewman,aclman)
  FILE **fp;
  PartyEntry partyagent;
  ContextEntry contextagent;
  ViewEntry viewagent;
  AclEntry aclagent;
  PartyEntry partyman;
  ContextEntry contextman;
  ViewEntry viewman;
  AclEntry aclman;
  {
  int lineLen;        /* local variables */
  char line[BUFSIZE];
  char word[BUFSIZE];
  int state = 0;
  int recnumber = 0;
  int filenumber = 0;

  MINclearall(partyagent,contextagent,viewagent,aclagent); /* Clear all records from the agent */
  MINclearall(partyman,contextman,viewman,aclman);         /* Clear all records from the manager */

  for (filenumber = 0; filenumber <= 7; filenumber++)  /* 8 files */
   {
   state = 1;

   recnumber = 0;
   while ((lineLen = MINreadln( fp[filenumber], line)) != -1) /* not EOF */
    {
    if (lineLen < 255)
     switch(filenumber)
       {
       case 0:MINfillparty(partyagent,line, &state, &recnumber, lineLen); /* fills agent parties */
              break;
       case 1:MINfillcontext(contextagent,line, &state, &recnumber, lineLen);  /* fills agent contexts */
              break;
       case 2:MINfillview(viewagent,line, &state, &recnumber, lineLen);  /* fills agent views */
              break;
       case 3:MINfillacl(aclagent,line, &state, &recnumber, lineLen);  /* fills agent acls */
              break;
       case 4:MINfillparty(partyman  ,line, &state, &recnumber, lineLen);  /* fills manager parties */
              break;
       case 5:MINfillcontext(contextman,line, &state, &recnumber,lineLen); /* fills manager contexts */
              break;
       case 6:MINfillview(viewman,line, &state, &recnumber,lineLen); /* fills manager views */
              break;
       case 7:MINfillacl(aclman,line, &state, &recnumber,lineLen); /* fills manager acls */
              break;
       } /* end switch */
    } /* end while loop */
   } /* end for loop */
 }




/* --------------------------------------------------------------------
 * Function  : AddViewRecordCmu
 * Arguments : fp, a filepointer (IN/OUT)
 *           : view, pointer of viewentrystruc (IN/OUT)
 * Purpose   : This function adds a record to cmu view file
 * Remarks   :
 * --------------------------------------------------------------------
 */
void MINAddViewRecordToCmu(fp,view)
  FILE *fp;
  ViewEntryStruc view;
  { /* Store view record as cmu format with tabs*/
  fprintf (fp,"\n");
  fprintf (fp,"%-3d\t%-20s\t%-10s\t%s\n",view.viewIndex,view.viewSubtree,view.viewMask,view.viewType);
  }


/* --------------------------------------------------------------------
 * Function  : MINAddAclRecordCmu
 * Arguments : fp, a filepointer (IN/OUT)
 *           : acl, pointer of aclentrystruc (IN/OUT)
 * Purpose   : This function adds a record to cmu acl file
 * Remarks   :
 * --------------------------------------------------------------------
 */
void MINAddAclRecordToCmu(fp,acl)
  FILE *fp;
  AclEntryStruc acl;
  { /* Store acl record as cmu format */

  fprintf (fp,"\n"); /* New line */
  fprintf (fp,"%-26s\t%-26s\t%-20s\t%s\n",acl.aclTarget,acl.aclSubject,acl.aclResources,acl.aclPrivileges);/* add record */
  }


/* --------------------------------------------------------------------
 * Function  : MINAddContextRecordCmu
 * Arguments : fp, a filepointer (IN/OUT)
 *           : context, pointer of contextentrystruc (IN/OUT)
 * Purpose   : This function adds a record to cmu context file
 * Remarks   :
 * --------------------------------------------------------------------
 */
void MINAddContextRecordToCmu(fp,context)
  FILE *fp;
  ContextEntryStruc context;
  { /* Store context record as cmu format */
  if (strlen(context.contextLocalEntity) == 0)
     strcpy(context.contextLocalEntity,"Null");
  fprintf (fp,"\n");
  fprintf (fp,"%-20s\t%s\n",context.contextName, context.contextIdentity);
  fprintf (fp,"%-20d\t%-33s\t%s\n",context.contextViewIndex,context.contextLocalEntity,context.contextLocalTime);
  fprintf (fp,"%-20s\t%-33s\t%s\n",context.contextProxyDstParty, context.contextProxySrcParty, context.contextProxyContext);
  }



/* --------------------------------------------------------------------
 * Function  : MINAddPartyRecordCmu
 * Arguments : fp, a filepointer (IN/OUT)
 *           : party, pointer of partyentrystruc (IN/OUT)
 * Purpose   : This function adds a record to the cmu party file
 * Remarks   :
 * --------------------------------------------------------------------
 */
void MINAddPartyRecordToCmu(fp,party)
 FILE *fp;
 PartyEntryStruc party;
 { /* Store party record as cmu format */
 int privPublicLength = 0;            /* Local variable */
 int authPublicLength = 0;
 char privPublicLengthstr[PRIVLEN] = "";
 char authPublicLengthstr[AUTHLEN] = "";

 privPublicLength = strlen(party.partyPrivPublic); /* calculate length PrivPublic */
 if (privPublicLength == 0)
   strcpy(privPublicLengthstr,"Null"); /* length == 0 */
 else sprintf(privPublicLengthstr,"%d",privPublicLength); /* convert integer to string */

 authPublicLength = strlen(party.partyAuthPublic); /* calculate length AuthPublic */
 if (authPublicLength == 0)
   strcpy(authPublicLengthstr,"Null"); /* length == 0 */
 else sprintf(authPublicLengthstr,"%d",authPublicLength); /* convert integer to string */

 
 fprintf (fp,"\n");
 fprintf (fp,"%-33s\t%s\n",party.FriendbyName, party.partyIdentity); /* Party record */
 fprintf (fp,"%-33s\t%-33s\t%s\n",party.partyTDomain,party.IPaddress,party.partyTAddress);
 fprintf (fp,"%-33s\t%s\n",party.partyAuthProtocol,party.partyPrivProtocol);
 fprintf (fp,"%-3s\t%s\n",party.partyAuthLifeTime,party.partyMaxMessageSize);
 fprintf (fp,"%-33s\n",party.partyAuthClock);
 fprintf (fp,"%-33s\t%-33s\t%s\n",party.partyAuthPrivate,authPublicLengthstr,party.partyAuthPublic);
 fprintf (fp,"%-33s\t%-33s\t%s\n",party.partyPrivPrivate,privPublicLengthstr,party.partyPrivPublic);
}



/* --------------------------------------------------------------------
 * Function  : MINwritemanagerfiles
 * Arguments : fp, a filepointer (IN/OUT)
 *           : partyman, pointer of partyentry (IN/OUT)
 *           : contextman, pointer of contextentry (IN/OUT)
 *           : viewman, pointer of viewentry (IN/OUT)
 *           : aclman, pointer of aclentry (IN/OUT)
 * Purpose   : This function adds the party, context, acl and view records
 *           : in the cmu manager files.
 * Remarks   :
 * --------------------------------------------------------------------
 */
void MINwritemanagerfiles(fp,partyman,contextman,viewman,aclman,output_widget_name)  
FILE **fp; 
  PartyEntry partyman;
  ContextEntry contextman;
  ViewEntry viewman;
  AclEntry aclman;
  char *output_widget_name;
 {
 int i=0;
 char xIMtext[ maximum_string_length ];

 i = 0;
 strcpy( xIMtext , xIMStatusLine12 );
 while(i<lengthEntry) /* all manager parties records */
   {
   if (partyman[i].partyIdentity[0] != '\0') /* party record not empty */
     {
     strcat( xIMtext , ".\0" );
     xRESPrintStringnPause( output_widget_name , xIMtext , 1 );
     MINAddPartyRecordToCmu(fp[0],partyman[i]); 
     }
   i++;
   }
 i = 0;
 strcpy( xIMtext , xIMStatusLine13 );
 while(i<lengthEntry) /* all manager context records */
   {
   if (contextman[i].contextIdentity[0] != '\0') /* context record not empty */
   {
     strcat( xIMtext , ".\0" );
     xRESPrintStringnPause( output_widget_name , xIMtext , 1 );
     MINAddContextRecordToCmu(fp[1],contextman[i]);
   }
   i++;
 }
 i = 0;
 strcpy( xIMtext , xIMStatusLine14 );
 while(i<lengthEntry) /* all manager view records */
   {
   if (viewman[i].viewIndex != -1) /* view record not empty */
   {
     strcat( xIMtext , ".\0" );
     xRESPrintStringnPause( output_widget_name , xIMtext , 1 );
     MINAddViewRecordToCmu(fp[2],viewman[i]);
   }
   i++;
   } 
 i = 0;
 strcpy( xIMtext , xIMStatusLine15 );
 while(i<lengthEntry) /* all manager acl records */
   {
   if (aclman[i].aclTarget[0] != '\0') /* acl record not empty */
   {
     strcat( xIMtext , ".\0" );
     xRESPrintStringnPause( output_widget_name , xIMtext , 1 );
     MINAddAclRecordToCmu(fp[3],aclman[i]);
   }
   i++;
   }
 }


/* --------------------------------------------------------------------
 * Function  : MINwritemanagerfiles
 * Arguments : fp, a filepointer (IN/OUT)
 * Purpose   : This function close the agent files 
 * Remarks   :
 * --------------------------------------------------------------------
 */
MINcloseagentfiles(fp) 
 FILE **fp; 
 {
 int i;
 for (i = 4; i < 8; i++) fclose(fp[i]); 
 }

 
/* --------------------------------------------------------------------
 * Function  : MINemptyfile
 * Arguments : fp, a filepointer (IN/OUT)
 *           : Inifile, array of chars (IN)
 * Purpose   : This function adds the cmu headers (stored in the .ini files)
 *           : in the manager cmu files 
 * Remarks   :
 * --------------------------------------------------------------------
 */
MINemptyfile(fp,IniFile,output_widget_name)
 FILE *fp;
 char *IniFile;
 char *output_widget_name;
   {
   FILE *finit;
   int buffer;
   char filename[FILELEN];
   char xIMtext[ maximum_string_length ];

   strcpy(filename,"");
   if( (char *) getenv( "SNMP_PARTYMIB_INITPATH" ) != NULL )
	strcpy( filename, (const char *) getenv( "SNMP_PARTYMIB_INITPATH" ) );
   strcat(filename,IniFile);
   if ((finit=fopen(filename,"r")) == (FILE *) NULL)
     {
      strcpy( xIMtext , "ERROR: Unable to open " );
      strcat( xIMtext , filename );
      strcat( xIMtext , " for reading.\0" );
      xRESPrintStringnPause( output_widget_name , xIMtext , 4 );
      exit(-1);
     }
   while ((buffer=getc(finit)) != EOF)
     {
     putc(buffer, fp);
     }
   fclose(finit);
   }
						 



/* --------------------------------------------------------------------
 * Function  : MINclosemanagerfiles
 * Arguments : fp, a filepointer (IN/OUT)
 * Purpose   : This function close all manager files 
 * Remarks   :
 * --------------------------------------------------------------------
 */
MINclosemanagerfiles(fp) 
 FILE **fp;
 {
 int i;
 for (i = 0; i < 4; i++) fclose(fp[i]);
 }
 
