/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:	snmpInitManSys.c
 *
 * Date:	May 1994
 *
 * Purpose:	Initialize partymib configuration files.
 *
 * Author:	Michel van Ooijen & Edward Veluwenkamp
 *		Peter Peters
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */

#include <snmpInitManager.h>


/* -----------------------------------------------------------------------
 *  Function : intro
 *  Arguments: hostnameM, hostname for management application (OUT).
 *             ipaddressM, internet address for management application (OUT).
 *             hostnameA, hostname for agent application (OUT).
 *             ipaddressA, internet address for agent application (OUT).
 *  Purpose  : intro on screen and user interaction to get hostname manager
 *             and hostname agent 
 *  Remarks  :   
 * -----------------------------------------------------------------------
 */
MINintro(ipaddressM,hostnameM,ipaddressA,hostnameA)
 char *ipaddressM,*hostnameM,*ipaddressA,*hostnameA;
{

      	system( "clear" );
      	fprintf( stdout,
		"\n ************************************************************************\n");
     	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );

	fprintf( stdout, " * ---------------\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout,
	    	" * This program creates initial partymib *.conf-files for applications,\t*\n" );
	fprintf( stdout,
	 	" * acting in the manager role, belonging to the cmu-snmp-v2.1 or \t*\n" );
	fprintf( stdout, " * ut-snmp-v2 packages.\t\t\t\t\t\t\t*\n");
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\tSuccess, \t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\tsnmp@cs.utwente.nl\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " * Additional remark:\t\t\t\t\t\t\t*\n");
	fprintf( stdout, " *\t%% setenv SNMP_PARTYMIB_PATH ./ \t\t# path for partyfiles\t*\n");
	fprintf( stdout, " *\t%% setenv SNMP_PARTYMIB_INITPATH ./etc/\t# path to init-files\t*\n");
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout,
		" ************************************************************************\n\n");
	fprintf( stdout, " Specify user-definable data:\n\n" );
  	MISipaddresshostname(0,ipaddressM,hostnameM);
  	MISipaddresshostname(1,ipaddressA,hostnameA);
	fprintf( stdout, "\n Status:\n\n" );
}

/* -----------------------------------------------------------------------
 *  Function:  printpartyentry
 *  Arguments: partyuser, initial pointer to partyentry (IN/OUT)
 *             count, initial integer counter (IN)
 *  Purpose:   prints partyuser[count] entry on the screen
 *  Remarks:   
 * -----------------------------------------------------------------------
*/
MINprintpartyentry(partyuser,count)
 PartyEntry partyuser;
 int        count;
 {
  if ((count > lengthEntry) || (count < 0)) fprintf(stdout,"ERROR wrong count\n");
  else
    if (partyuser[count].partyIdentity[0] != '\0')
     {
      fprintf(stdout,"\nPartyEntry[%d]\n\n",count);
      fprintf(stdout,"FriendbyName       : %s\n",partyuser[count].FriendbyName);
      fprintf(stdout,"partyIdentity      : %s\n",partyuser[count].partyIdentity);
      fprintf(stdout,"partyTDomain       : %s\n",partyuser[count].partyTDomain);
      fprintf(stdout,"IPaddress          : %s\n",partyuser[count].IPaddress);
      fprintf(stdout,"partyTAddress      : %s\n",partyuser[count].partyTAddress); 
      fprintf(stdout,"partyMaxMessageSize: %s\n",partyuser[count].partyMaxMessageSize);
      fprintf(stdout,"partyAuthProtocol  : %s\n",partyuser[count].partyAuthProtocol);
      fprintf(stdout,"partyAuthClock     : %s\n",partyuser[count].partyAuthClock);
      fprintf(stdout,"partyAuthPrivate   : %s\n",partyuser[count].partyAuthPrivate);
      fprintf(stdout,"partyAuthLifeTime  : %s\n",partyuser[count].partyAuthLifeTime);
      fprintf(stdout,"partyPrivProtocol  : %s\n",partyuser[count].partyPrivProtocol);
      fprintf(stdout,"partyPrivPrivate   : %s\n",partyuser[count].partyPrivPrivate);
     }
 }
 
 
 

/* -----------------------------------------------------------------------
 *  Function:  printcontextentry
 *  Arguments: contextuser, initial pointer to contextentry (IN/OUT)
 *	       count, initial integer counter (IN)
 *  Purpose:   prints contextuser[count] entry on the screen  
 *  Remarks:   
 * -----------------------------------------------------------------------
*/  
MINprintcontextentry(contextuser,count)
 ContextEntry contextuser;
 int          count;
 {
  if ((count > lengthEntry) || (count < 0)) fprintf(stdout,"ERROR wrong count\n");
  else
    if (contextuser[count].contextIdentity[0] != '\0')
     {
      fprintf(stdout,"\nContextEntry[%d]\n\n",count);
      fprintf(stdout,"contextName:%s\n",contextuser[count].contextName);
      fprintf(stdout,"contextIdentity:%s\n",contextuser[count].contextIdentity);
      fprintf(stdout,"IPaddress:%s\n",contextuser[count].IPaddress);
      fprintf(stdout,"contextViewIndex:%d\n",contextuser[count].contextViewIndex);
      fprintf(stdout,"contextLocalEntity:%s\n",contextuser[count].contextLocalEntity);
      fprintf(stdout,"contextLocalTime:%s\n",contextuser[count].contextLocalTime);
      fprintf(stdout,"contextProxyDstParty:%s\n",contextuser[count].contextProxyDstParty);
      fprintf(stdout,"contextProxySrcParty:%s\n",contextuser[count].contextProxySrcParty);
      fprintf(stdout,"contextProxyContext:%s\n",contextuser[count].contextProxyContext);
     }  
 }
 
 
/* -----------------------------------------------------------------------
 *  Function:  printaclentry 
 *  Arguments: acluser, initial pointer to aclentry (IN/OUT)
 *             count, initial integer counter (IN)
 *  Purpose:   prints acluser[count] entry on the screen  
 *  Remarks:   
 * -----------------------------------------------------------------------
*/ 
MINprintaclentry(acluser,count)
 AclEntry acluser;
 int       count;
 {
  if ((count > lengthEntry) || (count < 0)) fprintf(stdout,"ERROR wrong count\n");
  else
    if (acluser[count].aclTarget[0] != '\0')
     {   
      fprintf(stdout,"\nAclEntry[%d]\n\n",count);
      fprintf(stdout,"aclTarget:%s\n",acluser[count].aclTarget);
      fprintf(stdout,"aclSubject:%s\n",acluser[count].aclSubject);
      fprintf(stdout,"aclResources:%s\n",acluser[count].aclResources);
      fprintf(stdout,"aclPrivileges:%s\n",acluser[count].aclPrivileges);
     } 
 }


/* -----------------------------------------------------------------------
 *  Function:  printviewentry
 *  Arguments: viewuser, initial pointer to viewentry (IN/OUT)
 *             count, initial integer counter (IN)
 *  Purpose:   prints viewuser[count] entry on the screen  
 *  Remarks:     
 *  Remarks:   
 * -----------------------------------------------------------------------
*/
MINprintviewentry(viewuser,count)
 ViewEntry viewuser;
 int        count;
 {
  if ((count > lengthEntry) || (count < 0)) fprintf(stdout,"ERROR wrong count\n");
  else
    if (viewuser[count].viewIndex != -1)
     {      
      fprintf(stdout,"\nViewEntry[%d]\n\n",count);
      fprintf(stdout,"viewIndex:%d\n",viewuser[count].viewIndex);
      fprintf(stdout,"viewSubtree:%s\n",viewuser[count].viewSubtree);
      fprintf(stdout,"viewMask:%s\n",viewuser[count].viewMask);
      fprintf(stdout,"viewType:%s\n",viewuser[count].viewType);
     }
 }
  


/* -----------------------------------------------------------------------
 *  Function:  sortpartyentries
 *  Arguments: partyuser, initial pointer to partyentry (IN/OUT)
 *  Purpose:   sorts PartyEntries through deleting all entries where
 *             partyIdentity[0] = '\0' 
 *  Remarks:   
 * -----------------------------------------------------------------------
*/
MINsortpartyentries(partyuser)
 PartyEntry partyuser;
 {
  int i=0,j=lengthEntry;
  
  while(i<lengthEntry)
   {
    if ((partyuser[i].partyIdentity[0] != '\0') && (j < i))
     {
      memcpy(&partyuser[j], &partyuser[i],(sizeof(partyuser[i])));
      partyuser[i].partyIdentity[0] = '\0';
      i=j;
      j=lengthEntry;
     }
    if ((partyuser[i].partyIdentity[0] == '\0') && (j > i)) j=i;
    i++;
   }
 }
 
 
/* -----------------------------------------------------------------------
 *  Function:  sortaclentries
 *  Arguments: acluser, initial pointer to aclentry (IN/OUT)
 *  Purpose:   sorts AclEntries through deleting all entries where
 *             aclTarget[0] = '\0' 
 *  Remarks:   
 * -----------------------------------------------------------------------
*/ 
MINsortaclentries(acluser)
 AclEntry acluser;
 {
  int i=0,j=lengthEntry;

  while(i<lengthEntry)
   {
    if ((acluser[i].aclTarget[0] != '\0') && (j < i))
     {
      memcpy(&acluser[j], &acluser[i],(sizeof(acluser[i])));
      acluser[i].aclTarget[0] = '\0';
      i=j;
      j=lengthEntry;
     }
    if ((acluser[i].aclTarget[0] == '\0') && (j > i)) j=i;
    i++;
   }
 }
 
 
/* -----------------------------------------------------------------------
 *  Function:  sortcontextentries
 *  Arguments: contextuser, initial pointer to contextentry (IN/OUT)
 *  Purpose:   sorts ContextEntries through deleting all entries where
 *             contextIdentity[0] = '\0' 
 *  Remarks:   
 * -----------------------------------------------------------------------
*/  
MINsortcontextentries(contextuser)
 ContextEntry contextuser;
 {
  int i=0,j=lengthEntry;
  
  while(i<lengthEntry)
   {
    if ((contextuser[i].contextIdentity[0] != '\0') && (j < i))
     {
      memcpy(&contextuser[j], &contextuser[i],(sizeof(contextuser[i])));
      contextuser[i].contextIdentity[0] = '\0';
      i=j;
      j=lengthEntry;
     }
    if ((contextuser[i].contextIdentity[0] == '\0') && (j > i)) j=i;
    i++;
   }
 }



/* -----------------------------------------------------------------------
 *  Function:  sortviewentries
 *  Arguments: viewuser, initial pointer to viewentry (IN/OUT)
 *  Purpose:   sorts ViewEntries through deleting all entries where
 *             viewIndex = -1
 *  Remarks:   
 * -----------------------------------------------------------------------
*/
MINsortviewentries(viewuser)
 ViewEntry viewuser;
 {
  int i=0,j=lengthEntry;

  while(i<lengthEntry)
   {
    if ((viewuser[i].viewIndex != -1) && (j < i))
     {
      memcpy(&viewuser[j], &viewuser[i],(sizeof(viewuser[i])));
      viewuser[i].viewIndex = -1;
      i=j;
      j=lengthEntry;
     }
    if ((viewuser[i].viewIndex == -1)&& (j > i)) j=i;
    i++;
   }
 }



/* -----------------------------------------------------------------------
 *  Function:  removedoublepartyentries
 *  Arguments: partyagent, initial pointer to partyentry (IN/OUT)
 *             partyman, initial pointer to partyentry (IN/OUT)
 *  Purpose:   removes all entries in the manager party that have the same 
 *             partyIdentity as an entry in the agent party.
 *  Remarks:   
 * -----------------------------------------------------------------------
*/
MINremovedoublepartyentries(partyagent,partyman)
 PartyEntry partyagent,partyman;
 {
  int i=0,j;

  while((i<lengthEntry) && (partyman[i].partyIdentity[0]  !='\0'))
   {
    j=0;
    while((j<lengthEntry) && (partyagent[j].partyIdentity[0] !='\0'))
     {
      if(strcmp(partyman[i].partyIdentity,partyagent[j].partyIdentity) == 0)
         partyman[i].partyIdentity[0] = '\0';
      j++;
     }
    i++;
   }
 }



/* -----------------------------------------------------------------------
 *  Function:  removedoubleaclentries 
 *  Arguments: aclagent, initial pointer to aclentry (IN/OUT)
 *             aclman, initial pointer to aclentry (IN/OUT)
 *  Purpose:   removes all entries in the manager Acl that have the same 
 *             aclTarget, aclSubject and aclResource as an entry in the agent
 *             Acl. 
 *  Remarks:   
 * -----------------------------------------------------------------------
*/ 
MINremovedoubleaclentries(aclagent,aclman) 
 AclEntry aclagent,aclman;
 {
  int i=0,j;
  
  while((i<lengthEntry) && (aclman[i].aclTarget[0]  !='\0'))
   {
    j=0;
    while((j<lengthEntry) && (aclagent[j].aclTarget[0] !='\0'))
     {
      if((strcmp(aclman[i].aclTarget,aclagent[j].aclTarget) == 0) &&
         (strcmp(aclman[i].aclSubject,aclagent[j].aclSubject) == 0) &&
         (strcmp(aclman[i].aclResources,aclagent[j].aclResources) == 0))
             aclman[i].aclTarget[0] = '\0';
      j++;
     }
    i++;
   }
 }
 

/* -----------------------------------------------------------------------
 *  Function:  removedoublecontext_viewentries
 *  Arguments: contextagent, initial pointer to contextentry (IN/OUT)
 *             contextman, initial pointer to contextentry (IN/OUT)
 *             viewagent, initial pointer to viewentry (IN/OUT)
 *             viewman, initial pointer to viewentry (IN/OUT)
 *  Purpose:   removes all entries in the manager Context that have the same
 *             contextIdentity as an entry in the agent Context.
 *             The entries in the manager View with an ViewIndex equal
 *             to contextViewIndex in a removed context of the manager
 *             will be deleted
 *  Remarks:   
 * -----------------------------------------------------------------------
*/
MINremovedoublecontext_viewentries(contextagent,contextman,viewagent,viewman)
 ContextEntry contextagent,contextman;
 ViewEntry    viewagent,viewman;
 {
  int i=0,j,k;
  
  while((i<lengthEntry) && (contextman[i].contextIdentity[0]  !='\0'))
   {
    j=0;
    while((j<lengthEntry) && (contextagent[j].contextIdentity[0] !='\0'))
     {
      if (strcmp(contextman[i].contextIdentity,contextagent[j].contextIdentity) == 0)
          {
            for(k=0;k<lengthEntry;k++)
             {
              if ((viewman[k].viewIndex != -1) &&
                 (contextman[i].contextViewIndex == viewman[k].viewIndex))
                  {
                    viewman[k].viewIndex = -1;  
                  }
             }
            contextman[i].contextIdentity[0] = '\0';
          }
       j++;
     }
    i++;
   }
 } 



/* -----------------------------------------------------------------------
 *  Function:  copypartyagenttoman 
 *  Arguments: partyagent, initial pointer to partyentry (IN/OUT)
 *             partyman, initial pointer to partyentry (IN/OUT)
 *  Purpose:   copy Party of agent to manager Party.
 *  Remarks:   
 * -----------------------------------------------------------------------
*/
MINcopypartyagenttoman(partyagent,partyman)
 PartyEntry partyagent,partyman;
 {
  int i=0,j=0;
  
  while ((i<(lengthEntry-1)) && (j < (lengthEntry-1)))
   {
    while ((partyman[i].partyIdentity[0] != '\0') && (i < (lengthEntry-1))) i++;
    while ((partyagent[j].partyIdentity[0] == '\0') && (j < (lengthEntry-1))) j++;     
    if ((i<(lengthEntry-1)) && (j < (lengthEntry-1)))
      {
       memcpy(&partyman[i], &partyagent[j],(sizeof(partyman[i])));
       j++;
      }
   }
 }
 
/* -----------------------------------------------------------------------
 *  Function:  copyaclagenttoman
 *  Arguments: aclagent, initial pointer to aclentry (IN/OUT)
 *             aclman, initial pointer to aclentry (IN/OUT)
 *  Purpose:   copy Acl of agent to manager Acl.
 *  Remarks:   
 * -----------------------------------------------------------------------
*/
MINcopyaclagenttoman(aclagent,aclman)
 AclEntry aclagent,aclman;
 {
  int i=0,j=0;
  
   while ((i<(lengthEntry-1)) && (j < (lengthEntry-1)))
   {
    while ((aclman[i].aclTarget[0] != '\0') && (i < (lengthEntry-1))) i++;
    while ((aclagent[j].aclTarget[0] == '\0') && (j < (lengthEntry-1))) j++;
    if ((i<(lengthEntry-1)) && (j < (lengthEntry-1)))
      {
       memcpy(&aclman[i],&aclagent[j],(sizeof(aclman[i])));
       j++;
      }
   }
 }


/* -----------------------------------------------------------------------
 *  Function:  getfreeviewindex 
 *  Arguments: viewman, initial pointer to viewentry (IN/OUT)
 *  Purpose:   Gets a notused viewindex number.  
 *  Remarks:   
 * -----------------------------------------------------------------------
*/
int MINgetfreeviewindex(viewman)
 ViewEntry viewman;
 {
  int i=0,free=1;
  
  while ((i < lengthEntry) && (viewman[i].viewIndex != -1))
    {
     if (viewman[i].viewIndex >= free) free = viewman[i].viewIndex+1;
     i++;
    }
  return(free);
 }



/* -----------------------------------------------------------------------
 *  Function:  copycontext_viewagenttoman
 *  Arguments: contextagent, initial pointer to contextentry (IN/OUT)
 *             contextman, initial pointer to contextentry (IN/OUT)
 *             viewagent, initial pointer to viewentry (IN/OUT)
 *             viewman, initial pointer to viewentry (IN/OUT)
 *  Purpose:   copy Context and corresponding Views of agent to
 *             manager Context and view.
 *  Remarks:   
 * -----------------------------------------------------------------------
 */
MINcopycontext_viewagenttoman(contextagent,contextman,viewagent,viewman)
 ContextEntry contextagent,contextman;
 ViewEntry    viewagent,viewman;
 {
   int i=0,j=0,k=0,m=0,viewindex;
   
   while((i<(lengthEntry-1)) && (j <(lengthEntry-1)) && (m <(lengthEntry-1)))
    {
     while ((contextman[i].contextIdentity[0] != '\0') && (i < (lengthEntry-1))) i++;
     while ((contextagent[j].contextIdentity[0] == '\0') && (j < (lengthEntry-1))) j++;
     if ((i<lengthEntry) && (j < (lengthEntry-1)))
      {
       viewindex = MINgetfreeviewindex(viewman);
       memcpy (&contextman[i],&contextagent[j],(sizeof(contextman[i])));
       contextman[i].contextViewIndex = viewindex;
       for (k=0;k<(lengthEntry-1);k++)
        {
         if (viewagent[k].viewIndex == contextagent[j].contextViewIndex)
          {
           while ((viewman[m].viewIndex != -1) && (m < (lengthEntry-1))) m++;
           memcpy(&viewman[m], &viewagent[k],(size_t) (sizeof (viewman[m])));
           viewman[m].viewIndex = viewindex;
          }
        }
        j++;
      }
    }
 }
