/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:	xInitAgent.c
 *
 * Date:	May 1994
 *
 * Purpose:	The extension of the InitAgent for Athena including
 * 		communication between application and resource
 *
 * Author:	Peter Peters
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
 

#include <xInitAgent.h>
#include <snmpInitAgent.h>

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xAriSetDefaultsInitAgent
 * Argument: current_widget(in) , client_data(in) , call_data(in)
 * resource: xAriSetDefaultsInitAgentCB( *hostmanager *hostagent *portmanager *portagent *maxmessize *statustext )
 * purpose : default-button action, loads defaults into widgets
 * remarks : if use as c-function(not resource code) put resource argument in the client_data as string
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
void xAriSetDefaultsInitAgent( current_widget , client_data , call_data )
   Widget current_widget;
   XtPointer client_data;
   XtPointer call_data;
{
  char hostmanager[ maximum_string_length ] , hostagent[ maximum_string_length ] , portmanager[ maximum_string_length ];
  char portagent[ maximum_string_length ] , messagesize[ maximum_string_length ] , outputtext[ maximum_string_length ];
  char statusline[ maximum_string_length ] , hostname[ ARLEN ]; 
  int status1 , status2 , status3 , status4 , status5 , status0;
  Widget WhostM , WhostA , WportM , WportA , Wmesss;

  xRESInitVariables( current_widget );
  status0 = xRESGetnString( 0 , client_data , hostmanager );
  status1 = xRESGetnString( 1 , client_data , hostagent );
  status2 = xRESGetnString( 2 , client_data , portmanager );
  status3 = xRESGetnString( 3 , client_data , portagent );
  status4 = xRESGetnString( 4 , client_data , messagesize );
  status5 = xRESGetnString( 5 , client_data , outputtext );
  if( status5 == Ok_Status && status0 == Ok_Status  && status1 == Ok_Status && status2 == Ok_Status && status3 == Ok_Status && status4 == Ok_Status )
  {
    WhostM = WcFullNameToWidget( current_widget , hostmanager );
    WhostA = WcFullNameToWidget( current_widget , hostagent );
    WportM = WcFullNameToWidget( current_widget , portmanager );
    WportA = WcFullNameToWidget( current_widget , portagent );
    Wmesss = WcFullNameToWidget( current_widget , messagesize );
    if( WhostM != NULL && WhostA != NULL && WportM != NULL && WportA != NULL && Wmesss != NULL )
    {
      if( gethostname( hostname , ARLEN ) != 0 ) hostname[0] = '\0';
      WcSetValueFromString( WhostM , "value" , hostname );
      WcSetValueFromString( WhostA , "value" , hostname );
      WcSetValueFromString( WportM , "value" , "162" );
      WcSetValueFromString( WportA , "value" , "161" );
      WcSetValueFromString( Wmesss , "value" , "484" );
      xRESPrintString( outputtext , xIAStatusLine11 );
    }
    else
    {
      strcpy( statusline , pErrorMessage5 );
      xRESPrintString( outputtext , statusline );
    }
  }
  else
  {
    strcpy( statusline , pErrorMessage5 );
    xRESPrintString( outputtext , statusline );
  }
}

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xAriStartInitAgent
 * Argument: current_widget(in) , client_data(in) , call_data(in)
 * resource: xAriStartInitAgentCB( ( *hostmanager *hostagent *portmanager *portagent *maxmessize *statustext )
 * purpose : Starts init process, under ok-button
 * remarks : if use as c-function(not resource code) put resource argument in the client_data as string
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
void xAriStartInitAgent( current_widget, client_data, call_data )
  Widget current_widget;
  XtPointer client_data;
  XtPointer call_data;
{
  Widget widget_host_manager , widget_host_agent , widget_port_agent , widget_port_manager , widget_mess_size;
  char output_widget_name[ maximum_string_length ];
  char agent_host_value[ maximum_string_length ] , manager_host_value[ maximum_string_length ] , 
       agent_port_value[ maximum_string_length ] , manager_port_value[ maximum_string_length ] ,
       maximum_messize_value[ maximum_string_length ];
  char substring_manager_host[ maximum_string_length ] , substring_agent_host[ maximum_string_length ] ,
       substring_manager_port[ maximum_string_length ] , substring_agent_port[ maximum_string_length ] ,
       substring_mess_size[ maximum_string_length ];
  int maximum_mess_size , ret_stat_maxmes , ret_stat_portman , ret_stat_portagn;
  char hostname[ ARLEN ];
  FILE *fp;
  char filename[ FILENAMELEN ] = "";
  PartyEntry party1;  /* First party */
  PartyEntry party2;  /* Second party */
  ContextEntry context; /* Context table */ 
  ViewEntry view;  /* viewtable */
  AclEntry acl; /* Acl table */
  char *cp , xIAtext[ maximum_string_length ];
  int  KindMessage= 0; /* NoAuth = 0, Auth = 1, md5 = 2 */

  if( xRESInitVariables( current_widget ) == Fault_Status ) exit(1);
  /* get the values and test'em , first the hostnames, if empty fetch current hostname */
  if( xRESGetnString( 0 , client_data , &substring_manager_host[ 0 ] ) == Ok_Status &&
      xRESGetnString( 1 , client_data , &substring_agent_host[ 0 ] )   == Ok_Status &&
      xRESGetnString( 2 , client_data , &substring_manager_port[ 0 ] ) == Ok_Status &&
      xRESGetnString( 3 , client_data , &substring_agent_port[ 0 ] )   == Ok_Status &&
      xRESGetnString( 4 , client_data , &substring_mess_size[ 0 ] )    == Ok_Status &&
      xRESGetnString( 5 , client_data , &output_widget_name[ 0 ] )     == Ok_Status )
  {
    if( (widget_host_manager = WcFullNameToWidget( current_widget , substring_manager_host ) ) != NULL &&
        (widget_host_agent   = WcFullNameToWidget( current_widget , substring_agent_host ) )   != NULL &&
        (widget_port_manager = WcFullNameToWidget( current_widget , substring_manager_port ) ) != NULL &&
        (widget_port_agent   = WcFullNameToWidget( current_widget , substring_agent_port ) )   != NULL &&
        (widget_mess_size    = WcFullNameToWidget( current_widget , substring_mess_size ) )    != NULL )
    {
      strcpy( manager_host_value    , (String)XawDialogGetValueString( widget_host_manager ));
      strcpy( agent_host_value      , (String)XawDialogGetValueString( widget_host_agent ));
      strcpy( manager_port_value    , (String)XawDialogGetValueString( widget_port_manager ));
      strcpy( agent_port_value      , (String)XawDialogGetValueString( widget_port_agent ));
      strcpy( maximum_messize_value , (String)XawDialogGetValueString( widget_mess_size ));
      if( WcStrLen( manager_host_value) == 0 &&
          WcStrLen(agent_host_value ) == 0 &&
          WcStrLen(manager_port_value) == 0 && 
          WcStrLen(agent_port_value) == 0 &&
          WcStrLen(maximum_messize_value) == 0 )
      { 
	if( gethostname( hostname , ARLEN ) != 0 ) hostname[0] = '\0';
	strcpy( manager_host_value , hostname );
	strcpy( agent_host_value , hostname );
        strcpy( manager_port_value    , "162\0" );
        strcpy( agent_port_value      , "161\0" );
        strcpy( maximum_messize_value , "484\0" );
      }      
      /* range testing of inputted values & init tables */
      xRESPrintStringnPause( output_widget_name , xIAStatusLine0 , 1 );
      InitContextRecord(&context); /* Init Context record */
      InitViewRecord(&view); /* Init view record */
      InitAclRecord(&acl); /* Init acl record */
      InitPartyRecord(&party1); /* Agent party */
      InitPartyRecord(&party2); /* Manager party */
      xRESPrintStringnPause( output_widget_name , xIAStatusLine1 , 1 );
      ret_stat_maxmes  = xRESCheckIntegerString( maximum_messize_value );
      ret_stat_portman = xRESCheckIntegerString( manager_port_value );
      ret_stat_portagn = xRESCheckIntegerString( agent_port_value );
      if( SINgetipadrname( party1.IPaddress , agent_host_value ) == 0 &&
          SINgetipadrname( party2.IPaddress , manager_host_value ) == 0 &&
          ret_stat_maxmes  == Ok_Status && ret_stat_portman == Ok_Status && ret_stat_portagn == Ok_Status &&
  	  xRESCheckIntegerInRange( atoi( maximum_messize_value ) , 484 , 65507 ) == Ok_Status &&
	  xRESCheckIntegerInRange( atoi( manager_port_value ) , 0 , 65536 ) == Ok_Status &&
	  xRESCheckIntegerInRange( atoi( agent_port_value ) , 0 , 65536 ) == Ok_Status )
      {
        xRESPrintStringnPause( output_widget_name , xIAStatusLine2 , 1 );
	/* place portnumbers + messagesize in records */
	strcpy( party1.partyMaxMessageSize , maximum_messize_value );
	strcpy( party2.partyTAddress , manager_port_value );
	strcpy( party1.partyTAddress , agent_port_value );

        strcpy(party2.partyMaxMessageSize,party1.partyMaxMessageSize); /* manager and agent equal */
        strcpy(party1.partyIdentity,PartyIdentityStand);  /* standard OID */
        strcat(party1.partyIdentity,party1.IPaddress); /* strcat IP address */
        strcpy(party2.partyIdentity,PartyIdentityStand); /* standard OID */
        strcat(party2.partyIdentity,party1.IPaddress); /* strcat IP address */
        xRESPrintStringnPause( output_widget_name , xIAStatusLine3 , 1 );
	
        InitEmptyfile(PartyIniFile,Partyfile,agent_host_value,output_widget_name); /* Init CMU Header in 'hostname'_party.conf file */ 
        InitEmptyfile(ContextIniFile,Contextfile,agent_host_value,output_widget_name); /* Init CMU Header in 'hostname'_context.conf file */
        InitEmptyfile(AclIniFile,Aclfile,agent_host_value,output_widget_name); /* Init Header in 'hostname'_acl.conf file */
        InitEmptyfile(ViewIniFile,Viewfile,agent_host_value,output_widget_name); /* Init Header in 'hostname'_view.conf file */
        strcpy(filename,""); /* empty */
        /*if( getenv( "SNMP_PARTYMIB_PATH" ) != (char *) NULL )*/ 
        strcat( filename, ( const char *)getenv( "SNMP_PARTYMIB_PATH" ) );  
        strcat(filename,agent_host_value);
        strcat(filename,Partyfile);
        xRESPrintStringnPause( output_widget_name , xIAStatusLine4 , 1 );
        if ((fp=fopen(filename,"a+")) == (FILE *) NULL) /* opening for read/append */
        {
          strcpy( xIAtext , "ERROR: unable to open \0" );
          strcat( xIAtext , filename );
          strcat( xIAtext , " for saving.\0" );
	  xRESPrintStringnPause( output_widget_name , xIAtext , 4 );
          exit(-1);
        }
        party1.partyIndex = 1; /* Init counter partie 1 */			
        party2.partyIndex = 2; /* Init counter partie 2 */
        xRESPrintStringnPause( output_widget_name , xIAStatusLine5 , 1 );
	xRESPrintStringnPause( output_widget_name , xIAStatusLine8 , 1 );
        do
        {
          InitAssignPartyRecord(KindMessage,&party1); /* Agent party */
          InitAssignPartyRecord(KindMessage,&party2); /* Manager party */
   	  strcpy(party2.partyAuthPrivate,party1.partyAuthPrivate);
   	  strcpy(party2.partyPrivPrivate,party1.partyPrivPrivate);
          if (!UINpartyuser(KindMessage,party2.FriendbyName,agent_host_value,party1.FriendbyName,output_widget_name))
          {
            InitAddPartyRecordToCmu(fp, &party1 ); /* Store party Agent in the cmu party file */
            InitAddPartyRecordToCmu(fp, &party2 ); /* Store party Manager in the cmu party file */
            if (KindMessage < 2)
	    {
              InitAddViewcmufile(KindMessage,&view,agent_host_value,output_widget_name);
	      InitAddContextcmufile(KindMessage,&party1,&context,&view,agent_host_value,output_widget_name); 
	    }
            InitAddAclcmufile(KindMessage,&acl,&party1,&party2,&context,agent_host_value,output_widget_name); 
            party1.partyIndex = party1.partyIndex + 2; /* Counter next parties */			
            party2.partyIndex = party2.partyIndex + 2;
          }
          KindMessage ++; 
        } while (KindMessage < 3); /* six parties max input */
        xRESPrintStringnPause( output_widget_name , xIAStatusLine6 , 1 );
        fclose (fp);
        xRESPrintStringnPause( output_widget_name , xIAStatusLine7 , 4 );
        WcExitCB( current_widget , client_data , call_data );
      }
      else
        xRESPrintStringnPause( output_widget_name , pErrorMessage6 , 2 );
    }
    else
      xRESPrintStringnPause( output_widget_name , pErrorMessage11 , 2 );
  }
  else
    xRESPrintStringnPause( output_widget_name , pErrorMessage8 , 2 );
}
