/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:	xInitManager.c
 *
 * Date:	May 1994
 *
 * Purpose:	The file implements the communication between 
 *		xInitManager-application and resources
 *
 * Author:	Michel van Ooijen & Edward Veluwenkamp
 *		Peter Peters
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
 

/*including the files for prototype and vars */
#include <xInitManager.h>
#include <snmpInitManager.h>

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xAriStartInitManager
 * Argument: current_widget(in) , client_data(in) , call_data(in)
 * resource: xAriStartInitManagerCB( *hostmanager *hostagent *statustext )
 * purpose : starts the init-process under the ok-button
 * remarks : if use as c-function(not resource code) put resource argument in the client_data as string
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
void xAriStartInitManager( current_widget , client_data , call_data )
  Widget current_widget;
  XtPointer client_data;
  XtPointer call_data;
{
  /* widget adresses from the agent and manager */
  Widget widget_manager , widget_agent;
  /* the output widget from the client_data */
  char output_widget_name[ maximum_string_length ];
  /* values and ipadresses from agent & manager */
  char agent_value[ ADDRLEN ] , manager_value[ ADDRLEN ] , agent_ipadress[ ADDRLEN ] , 
       manager_ipadress[ ADDRLEN ];
  /* text for getting substring */
  char substring_manager[ maximum_string_length ] ,  substring_agent[ maximum_string_length ];
  /* status returns */
  int ret_stat_manager , ret_stat_agent , ret_stat_output;
  /* init manager variables */
  PartyEntry partyman;       /*  a pointer to the Partyentrys of the manager */
  PartyEntry partyagent;     /*  a pointer to the Partyentrys of the agent */
  ContextEntry contextman;   /*  a pointer to the Contextentrys of the manager */
  ContextEntry contextagent; /*  a pointer to the Contextentrys of the agent */
  ViewEntry viewman;         /*  a pointer to the Viewentrys of the manager */
  ViewEntry viewagent;       /*  a pointer to the Viewentrys of the agent */
  AclEntry aclman;           /*  a pointer to the Aclentrys of the manager */
  AclEntry aclagent;         /*  a pointer to the Aclentrys of the agent */
  FILE *fp[8];               /*  a filepointer array with eight filepointers, 4 agent files and 4 manager files */
  
  /* initializing the variables for output */
  if( xRESInitVariables( current_widget ) == Fault_Status ) exit(0);

  /* checking the parameterline, defragmenting into widget text */
  if( xRESGetnString( 0 , client_data , substring_manager) == Ok_Status  && 
      xRESGetnString( 1 , client_data , substring_agent )   == Ok_Status &&
      xRESGetnString( 2 , client_data , output_widget_name ) == Ok_Status )
  {
    /* get the widget-object-addresses from the manager and agent widget */
    if( ( widget_manager = WcFullNameToWidget( current_widget , substring_manager ) ) != NULL &&
        ( widget_agent   = WcFullNameToWidget( current_widget , substring_agent ) )   != NULL )
      {
        /* widgets exists, so get the values from the dialog boxes */
        strcpy( manager_value , (String)XawDialogGetValueString( widget_manager ) ); 
        strcpy( agent_value , (String)XawDialogGetValueString( widget_agent ) );
        /* check if both are empty or full */
        if( ( WcStrLen( manager_value ) == 0 && WcStrLen( agent_value ) == 0 ) ||
            ( WcStrLen( manager_value ) != 0 && WcStrLen( agent_value ) != 0 ) )
        {
          /* check if both are empty */
          if( WcStrLen( manager_value ) == 0 && WcStrLen( agent_value ) == 0 )
          {
            ret_stat_manager = gethostname( manager_value , ADDRLEN );
            ret_stat_agent   = gethostname( agent_value , ADDRLEN );
          }
          /* print a error message if one is empty and the other full, is not yet stable !!! */
          if( xRESPrintStringnPause( output_widget_name , xIMStatusLine0 , 1 ) == Fault_Status ) exit(-1);
          /* checking the IP-adresses */
          if( MISgetipadrname( manager_ipadress , manager_value ) == 0 &&
              MISgetipadrname( agent_ipadress , agent_value ) == 0 )
          {
            if( xRESPrintStringnPause( output_widget_name , xIMStatusLine1 , 1 ) == Fault_Status ) exit(1);
            /* Opening the files */
            if( MINopenmanagerfiles( fp , "r" , manager_value , output_widget_name ) != -1 &&
                MINopenagentfiles( fp , agent_value , output_widget_name ) != -1 )
            {
              if( xRESPrintStringnPause( output_widget_name , xIMStatusLine2 , 2 ) == Fault_Status ) exit(1);
              MINreadfiles( fp , partyagent , contextagent , viewagent , aclagent , partyman , contextman , viewman , aclman );
              MINcloseagentfiles( fp );
              MINclosemanagerfiles( fp );
              if( xRESPrintStringnPause( output_widget_name , xIMStatusLine3 , 1 ) == Fault_Status ) exit(1);
              MINremovedoublepartyentries( partyagent , partyman );
              MINsortpartyentries( partyman );
              if( xRESPrintStringnPause( output_widget_name , xIMStatusLine4 , 1 ) == Fault_Status ) exit(1);
              MINremovedoubleaclentries( aclagent , aclman );
              MINsortaclentries( aclman );
              if( xRESPrintStringnPause( output_widget_name , xIMStatusLine5 , 1 ) == Fault_Status ) exit(1);
              MINremovedoublecontext_viewentries( contextagent , contextman , viewagent , viewman );
              MINsortcontextentries( contextman );
	      MINsortviewentries( viewman );
              if( xRESPrintStringnPause( output_widget_name , xIMStatusLine6 , 1 ) == Fault_Status ) exit(1);
	      MINcopypartyagenttoman( partyagent , partyman );
	      MINcopyaclagenttoman( aclagent , aclman );
	      MINcopycontext_viewagenttoman( contextagent , contextman , viewagent , viewman );
	      if( xRESPrintStringnPause( output_widget_name , xIMStatusLine7 , 2 ) == Fault_Status ) exit(1);
	      if( MINopenmanagerfiles( fp , "w" , manager_value , output_widget_name ) != -1 )
	      {
	        if( xRESPrintStringnPause( output_widget_name , xIMStatusLine8 , 1 ) == Fault_Status ) exit(1);
		MINwritemanagerfiles( fp , partyman , contextman , viewman , aclman , output_widget_name );
		if( xRESPrintStringnPause( output_widget_name , xIMStatusLine9 , 1 ) == Fault_Status ) exit(1);
		MINclosemanagerfiles( fp );
		if( xRESPrintStringnPause( output_widget_name , xIMStatusLine10 , 4 ) == Fault_Status ) exit(1);
                WcExitCB( current_widget , client_data , call_data );
              }
              else
		xRESPrintStringnPause( output_widget_name , pErrorMessage5 , 2 );
            }
            else
               xRESPrintStringnPause( output_widget_name , pErrorMessage4 , 2 );
          }
          else
            xRESPrintStringnPause( output_widget_name , pErrorMessage2 , 2 );
        }
        else
          xRESPrintStringnPause( output_widget_name , pErrorMessage9 , 2 );
      }
      else
        xRESPrintStringnPause( output_widget_name , pErrorMessage1 , 2 );
  }
  else
    xRESPrintStringnPause( output_widget_name , pErrorMessage8 , 2 );
}

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xAriSetDefaultInitManager
 * Argument: current_widget(in)  , client_data(in) , call_data(in)
 * resource: xAriSetDefaultInitManagerCB( *hostmanager *hostagent )
 * purpose : action under the defaults-button, fetch defaults and put in boxes
 * remarks : if use as c-function(not resource code) put resource argument in the client_data as string
 *++++++++++++++++++++++++++++++++++++++++++++++
*/

void xAriSetDefaultInitManager( current_widget , client_data , call_data )
  Widget current_widget;
  XtPointer client_data , call_data;
{
  Widget widget_manager , widget_agent;
  char hostname[ ADDRLEN ];
  char substring_manager[ maximum_string_length ] ,  substring_agent[ maximum_string_length ];
  char output_widget_name[ maximum_string_length ];

  /* initialize the variables */
  if( xRESInitVariables( current_widget ) == Ok_Status )
  {
  /* checking the parameterline, defragmenting into widget text */
  if( xRESGetnString( 0 , client_data , substring_manager ) == Ok_Status && 
      xRESGetnString( 1 , client_data , substring_agent )   == Ok_Status &&
      xRESGetnString( 2 , client_data , output_widget_name ) == Ok_Status )
  {
    /* checking if the widget object path exists */
    if( ( widget_manager = WcFullNameToWidget( current_widget , substring_manager ) ) != NULL &&
        ( widget_agent   = WcFullNameToWidget( current_widget , substring_agent ) )   != NULL )
    {
      /* fetch the current hostname and put them into the value boxes */
      if( gethostname( hostname , ADDRLEN ) == 0 )
       {
         WcSetValueFromString( widget_manager , "value" , hostname );
         WcSetValueFromString( widget_agent , "value" , hostname );
         xRESPrintString( output_widget_name , xIMStatusLine11 );
       }
       else
          xRESPrintString( output_widget_name , pErrorMessage10 );
    }
    else
       xRESPrintString( output_widget_name , pErrorMessage11 );
  }
  else
    xRESPrintString( output_widget_name , pErrorMessage8 );
  }
}
