/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:	snmpInitManager.h
 *
 * Date:	May 1994
 *
 * Purpose:	Initialize partymib configuration files.
 *
 * Author:	Peter Peters
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */

#ifndef	SNMPINITMANAGER_H
#define SNMPINITMANAGER_H

#include <stdio.h>
#include <sys/types.h>
#include <errno.h>
#include <netinet/in.h>
#include <netdb.h>

#define  MAXSTRLEN       40
#define  FILELEN        100
#define  ARLEN          200
#define  BUFSIZE        512
#define  MAXAPPL        100 /* amount of applications on one system */
#define  lengthEntry    100
#define  MAXHOSTNAMELEN 256
#define  ADDRLEN         80 
#define  AUTHLEN         80 
#define  PRIVLEN         80 


typedef int	INTEGER;	/* local integer representation, int: 32 bits */

typedef struct 
  {
        char 		FriendbyName[MAXSTRLEN];
        char  		partyIdentity[MAXSTRLEN]; 	
	INTEGER		partyIndex;		/* position in the partyTable */
	char		partyTDomain[MAXSTRLEN];
        char            IPaddress[MAXSTRLEN];
	char		partyTAddress[MAXSTRLEN];
	char		partyMaxMessageSize[MAXSTRLEN];
	char		partyLocal; 			/* own or remote party */
	char		partyAuthProtocol[MAXSTRLEN];
	char		partyAuthClock[MAXSTRLEN];
	char		partyAuthPrivate[MAXSTRLEN];
	char		partyAuthPublic[MAXSTRLEN];
	char		partyAuthLifeTime[MAXSTRLEN];
	char		partyPrivProtocol[MAXSTRLEN];
	char		partyPrivPrivate[MAXSTRLEN];
	char		partyPrivPublic[MAXSTRLEN];
  } PartyEntryStruc;

typedef struct 
{
        char 		contextName[MAXSTRLEN];
        char		contextIdentity[MAXSTRLEN];
        char            IPaddress[MAXSTRLEN];
	INTEGER		contextIndex;	/* position in the contextTable */
	char 		contextLocal;	/* objects on this or remote system */
	INTEGER		contextViewIndex; /* could be a union structure */
	char		contextLocalEntity[MAXSTRLEN];
	char		contextLocalTime[MAXSTRLEN];
	char		contextProxyDstParty[MAXSTRLEN];
	char		contextProxySrcParty[MAXSTRLEN];
	char		contextProxyContext[MAXSTRLEN];
} ContextEntryStruc;

typedef struct 
{
	char		aclTarget[MAXSTRLEN];
	char		aclSubject[MAXSTRLEN];
	char		aclResources[MAXSTRLEN];
	char		aclPrivileges[MAXSTRLEN];
} AclEntryStruc;

typedef struct
{
	int		viewIndex;
	char   		viewSubtree[MAXSTRLEN];
	char 		viewMask[MAXSTRLEN];
	char		viewType[MAXSTRLEN];
} ViewEntryStruc;


typedef PartyEntryStruc PartyEntry[lengthEntry];
typedef ContextEntryStruc ContextEntry[lengthEntry]; 
typedef ViewEntryStruc ViewEntry[lengthEntry];
typedef AclEntryStruc AclEntry[lengthEntry]; 

#define TRUE 1
#define FALSE 0
#define IDENTITY  1
#define TRANSPORT 2
#define PROTOCOL  3
#define LIFETIME  4
#define CLOCK     5
#define AUTH      6
#define PRIV      7
  
extern char PARTYFILE[];
extern char VIEWFILE[];
extern char ACLFILE[];
extern char CONTEXTFILE[];
extern char PartyIdentityStand[];
extern char ContextIdentityStand[];
extern char VIEWINIFILE[];
extern char ACLINIFILE[];
extern char CONTEXTINIFILE[];
extern char PARTYINIFILE[];
extern MINprintpartyentry();
extern MINprintcontextentry();
extern MINprintaclentry();
extern MINprintviewentry();
extern char CMU_PATH[]; 

#endif	/* SNMPINITMANAGER_H */
