/* -----------------------------------------------------------------------
 *
 * Project: 	UT-SNMP
 *
 * Filename:    xsnmpTester.c
 *
 * Date:        June 1994
 *
 * Purpose:     Program for testing the UT_SNMP c-api.
 *
 *					The program reads an testscenario from an user-definable
 *				scenario file. The scenario consists of commands and 
 *				comments (!). The commands are executed. The testreponses
 *				produced by the command(s) are logged in an user-definable
 *				logfile. The comments are copied to the logfile, as well.
 *				
 * Author(s):   Antoinne Api `Olle'man, (Peter)^2s, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#include <apiSNMP.h>
#include <xsnmpTester.h>
#include <ResourceExtendSet.h>

#define mVb(p,n,c,v) \
	mVbPut(p,n,mVcPut(mVcNew(),VARBINDCHOICE_VALUE,mObPut(mObNew(),c,v)))


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrReadValue
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrReadValue( char *type, char *v, VarBind *vb, ObjectName *n )
{
	int s;
	AsnOid *oid;

	s = strlen(v);
	switch( (int) *type ) {
	case 'I' :
		vb = mVb( vb, n, OBJECTSYNTAX_INTEGER_VALUE, atoi(v) );
		break;
	case 'O' :
		mVb( vb, n, OBJECTSYNTAX_STRING_VALUE, mOcPut( mOcNew(s), v, s) );
		break;
	case 'B':
		ASCIItoOID( v, oid=mOiNew( 256 ) );
		mVb( vb, n, OBJECTSYNTAX_OBJECTID_VALUE, oid );
		break;	
	case 'S':
		mVb( vb, n, OBJECTSYNTAX_BIT_VALUE, mBiPut( mBiNew(s), v, s ) );
		break;
	case 'A':
		mVb( vb, n, OBJECTSYNTAX_IPADDRESS_VALUE, mIpPut( mIpNew(s), v, s ) );
		break;
	case 'C':
		mVb( vb, n, OBJECTSYNTAX_COUNTER_VALUE, atoi(v) );
		break;
	case 'G':
		mVb( vb, n, OBJECTSYNTAX_GAUGE_VALUE, atoi(v) );
		break;
	case 'K':
		mVb( vb, n, OBJECTSYNTAX_TIMETICKS_VALUE, atoi(v) );
		break;
	case 'Q':
		/* choiceId = OBJECTSYNTAX_ARBITRARY_VALUE ; */
		break;
	case 'N':
		/* choiceId = OBJECTSYNTAX_NSAPADDRESS_VALUE ; */
		break;
	case 'U':
		mVb( vb, n, OBJECTSYNTAX_BIG_COUNTER_VALUE, atoi(v) );
		break;
	case 'T':
		mVb( vb, n, OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE, atoi(v) );
		break;
	default:
		return FAILED;
	}
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrErrorTypes
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrErrorTypes( PDUInt mErrStat, FILE *f , char *output_widget_name , char **test_string )
{
	char errordescr[ maximum_string_length ] , *errorbuf;

	switch( (int) mErrStat ) 
	{
	case 0 : {
			fprintf( f, "noError");
			xRESPlaceStringinText( test_string , "noError\0" );
			break;
		}
	case TOOBIG : {
			fprintf( f, "tooBig");
			xRESPlaceStringinText( test_string , "tooBig\0" );
			break;
		}
	case NOSUCHNAME : {
			fprintf( f, "noSuchName");
			xRESPlaceStringinText( test_string , "noSuchName\0" );
			break;
		}
	case BADVALUE : {
			fprintf( f, "badValue");
			xRESPlaceStringinText( test_string , "badValue\0" );
			break;
		}
	case READONLY : {
			fprintf( f, "readOnly");
			xRESPlaceStringinText( test_string , "readOnly\0" );
			break;
		}
	case GENERR : {
			fprintf( f, "genErr");
			xRESPlaceStringinText( test_string , "genErr\0" );
			break;
		}
	case NOACCESS : {
			fprintf( f, "noAccess");
			xRESPlaceStringinText( test_string , "noAccess\0" );
			break;
		}
	case WRONGTYPE : {
			fprintf( f, "wrongType");
			xRESPlaceStringinText( test_string , "wrongType\0" );
			break;
		}
	case WRONGLENGTH : {
			fprintf( f, "wrongLength");
			xRESPlaceStringinText( test_string , "wrongLength\0" );
			break;
		}
	case WRONGENCODING : {
			fprintf( f, "wrongEncoding");
			xRESPlaceStringinText( test_string , "wrongEncoding\0" );
			break;
		}
	case WRONGVALUE : {
			fprintf( f, "wrongValue");
			xRESPlaceStringinText( test_string , "wrongValue\0" );
			break;
		}
	case NOCREATION : {
			fprintf( f, "noCreation");
			xRESPlaceStringinText( test_string , "noCreation\0" );
			break;
		}
	case INCONSISTENTVALUE : {
			fprintf( f, "inconsistentValue");
			xRESPlaceStringinText( test_string , "inconsistentValue\0" );
			break;
		}
	case RESOURCEUNAVAILABLE : {
			fprintf( f, "resourceUnavailable");
			xRESPlaceStringinText( test_string , "resourceUnavailable\0" );
			break;
		}
	case COMMITFAILED : {
			fprintf( f, "commitFailed");
			xRESPlaceStringinText( test_string , "commitFailed\0" );
			break;
		}
	case UNDOFAILED : {
			fprintf( f, "undoFailed");
			xRESPlaceStringinText( test_string , "undoFailed\0" );
			break;
		}
	case AUTHORIZATIONERROR : {
			fprintf( f, "authorizationError");
			xRESPlaceStringinText( test_string , "authorizationError\0" );
			break;
		}
	case NOTWRITABLE : {
			fprintf( f, "notWritable");
			xRESPlaceStringinText( test_string , "notWritable\0" );
			break;
		}
	case INCONSISTENTNAME : {
			fprintf( f, "inconsistentname");
			xRESPlaceStringinText( test_string , "inconsistentname\0" );
			break;
		}
	default : {
			fprintf( f, "errorNo %d", mErrStat );
			strcpy( errordescr , "errorNo %d\0" );
			if( ( errorbuf = (char *)malloc( (WcStrLen( errordescr ) + 20) * sizeof( char ) ) ) != NULL )
			{
			  sprintf( errorbuf , errordescr , mErrStat );
			  xRESPlaceStringinText( test_string , errorbuf );
			  free( errorbuf );
			}
			break;
		}
	}
	xRESPrintStringGo2End( output_widget_name , *test_string );
	return OK ;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWriteConfirm
 *
 * Arguments:	which, type of confirm
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserWriteConfirm( char *which, INTEGER mReqId, INTEGER mErrInd, 
			INTEGER mErrTyp, VarBindList *mVarBinds, FILE *f ,
			char *output_widget_name , char **test_string )
{
	char confirmdescr[ maximum_string_length] , *confirmbuf , TempFileName[ maximum_string_length ];
	FILE *temp_fp;
	struct stat statb;

	strcpy( confirmdescr , "snmp%sConfirm(\n\tINTEGER requestId = %d,\n\tINTEGER errorStatus = \0" );
	if( ( confirmbuf = (char *)malloc( ( WcStrLen( confirmdescr ) + WcStrLen( which ) + 20 ) * sizeof( char ) ) ) != NULL )
	{
		sprintf( confirmbuf , confirmdescr , which , mReqId );
		xRESPlaceStringinText( test_string , confirmbuf );
		free( confirmbuf );
	}
	xRESPrintStringGo2End( output_widget_name , *test_string );

 	fprintf( f, "snmp%sConfirm(\n", which );
	fprintf( f, "\tINTEGER requestId = %d,\n", mReqId );
	fprintf( f, "\tINTEGER errorStatus = " );

	MngrErrorTypes( mErrTyp, f , output_widget_name , test_string ); (void) fprintf( f, ",\n" );

	strcpy( confirmdescr , "\n\tINTEGER errorIndex = %d,\n\tVARLIST variableValues =\n\t\0" );
	if( ( confirmbuf = (char *)malloc( ( WcStrLen( confirmdescr ) + 20 + 2 ) * sizeof( char ) ) ) != NULL )
	{
		sprintf( confirmbuf , confirmdescr , which , mReqId );
		xRESPlaceStringinText( test_string , confirmbuf );
		free( confirmbuf );
	}
	xRESPrintStringGo2End( output_widget_name , *test_string );

	fprintf( f, "\tINTEGER errorIndex = %d,\n", mErrInd );
	fprintf( f, "\tVARLIST variableValues =\n" );
	fprintf( f, "\t" );

	CreateTempFileName( TempFileName );
	if( (temp_fp = fopen( TempFileName , "w" ) ) != NULL )
	{
		PrintVarBindList( temp_fp , mVarBinds , 8 );
		fclose( temp_fp );
		stat( TempFileName , &statb );
		if( (temp_fp = fopen( TempFileName , "r" ) ) != NULL )
		{
			if( (confirmbuf = (char *)malloc( (unsigned)statb.st_size + 1 ) ) != NULL )
			{
				fread( confirmbuf , sizeof( char ) , statb.st_size , temp_fp );
				confirmbuf[ statb.st_size ] = '\0';
				xRESPlaceStringinText( test_string , confirmbuf );
				free( confirmbuf );
			}
			else
			{
				xRESPlaceStringinText( test_string , pWarningMessage3 );
			}
			fclose( temp_fp );
		}
		else
		{
			xRESPlaceStringinText( test_string , pWarningMessage3 );
		}
	}
	else
	{
		xRESPlaceStringinText( test_string , pWarningMessage3 );
	}
	xRESPlaceStringinText( test_string , "\n\0" );
	xRESPrintStringGo2End( output_widget_name , *test_string );

	PrintVarBindList( f, mVarBinds, 8 ) ;

	strcpy( confirmdescr , "\n);\n" );
	xRESPlaceStringinText( test_string , confirmdescr );
	xRESPrintStringGo2End( output_widget_name , *test_string );

	fprintf( f, "\n" );
	fprintf( f, ");\n" ); Flush ;

	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrHandleNet
 *
 * Arguments:	type, service type (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrHandleNet( SPTYPE mType, FILE *f , char *output_widget_name , char **test_string )
{
	INTEGER 	mReqId = 0, mErrInd = 0, mErrTyp = 0;
	DESCRIPTION error = "";
	VarBindList	*mVarBinds = mVlNew( );
	char errordescr[ maximum_string_length ] = "\0" , *errorbuf;

	switch( mType )
	{

	case GET_CNF:
	if( snmpGetConfirm( &mReqId, &mErrTyp, &mErrInd, &mVarBinds ) == OK ) 
		MngrUserWriteConfirm( "Get", mReqId, mErrInd, mErrTyp, mVarBinds, f , output_widget_name , test_string );
	break;

	case GETNEXT_CNF:
	if( snmpGetNextConfirm( &mReqId, &mErrTyp, &mErrInd, &mVarBinds ) == OK )
		MngrUserWriteConfirm( "GetNext", mReqId, mErrInd, mErrTyp, mVarBinds, f , output_widget_name , test_string );
	break;

	case GETBULK_CNF:
	if( snmpGetBulkConfirm( &mReqId, &mErrTyp, &mErrInd, &mVarBinds ) == OK )
		MngrUserWriteConfirm( "GetBulk", mReqId, mErrInd, mErrTyp, mVarBinds, f , output_widget_name , test_string );
	break;

	case SET_CNF:
	if( snmpSetConfirm( &mReqId, &mErrTyp, &mErrInd ) == OK )
		MngrUserWriteConfirm( "Set", mReqId, mErrInd, mErrTyp, mVarBinds, f , output_widget_name , test_string );
	break;

/*	case INFORM_CNF:
	if( snmpInformConfirm( &mReqId, &mErrTyp, &mErrInd ) == OK )
		MngrUserWriteConfirm( "Set", mReqId, mErrInd, mErrTyp, mVarBinds, f , output_widget_name , test_string );
	break;
*/
	case INFORM_CNF:
	if( snmpInformConfirm( &mReqId, &mErrTyp, &mErrInd, &mVarBinds ) == OK )
		MngrUserWriteConfirm( "Inform", mReqId, mErrInd, mErrTyp, NULL, f , output_widget_name , test_string );
	break;

	case ERR_IND:
	if( snmpErrorDescription( &mReqId, error ) == OK ) {
		fprintf( f, "\nsnmpErrorDescription(\n" );
		fprintf( f, "\tINTEGER requestId = %d,\n", mReqId );
		fprintf( f, "\tDESCRIPTION error = %s\n", error );
		fprintf( f, ");\n" );
		Flush;
		strcpy( errordescr , "\nsnmpErrorDescription(\n\tINTEGER requestId = %d,\n\tDESCRIPTION error = %s\n);\n\0" );

		if( ( errorbuf = (char *)malloc( ( strlen( errordescr ) + strlen( error ) + 40 + 2 ) * sizeof( char ) ) ) != NULL )
		{
			sprintf( errorbuf , errordescr , mReqId , error );
			xRESPlaceStringinText( test_string , errorbuf );
			free( errorbuf );
		}
		xRESPrintStringGo2End( output_widget_name , *test_string );
	}
	break;

	default: 
	break;
	}
	mVlFree( mVarBinds );
	return FALSE;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	mapV
 *
 * Arguments:	variableNames, list of variable names (IN)
 *				vbl, reference to actual vbl (OUT)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Put variables in variable binding list.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
mapV( char *vars, VarBindList *vbl )
{
	AsnOid *n;
	short i;
	char *p[100];

	for( i=0, p[i]=strtok(vars," \n"); p[i]; p[++i]=strtok(NULL," \n") );
	for( i=0; p[i]; i=i+1 ) {
	 	ASCIItoOID( p[i], (n=mOiNew(64)) );
		mVlPut( vbl, mVbPut( mVbNew(64), n, mVcPut( mVcNew(),
				VARBINDCHOICE_UNSPECIFIED,NULL)));
	}
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	mapValues
 *
 * Arguments:	variableNames, list of variable names and values (IN)
 *				vbl, reference to actual vbl (OUT)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Put variables and values in variable binding list.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
mapValues( char *vars, VarBindList *vbl )
{
	AsnOid *n;
	short i;
	char *p[100];
	VarBind *vb;

	for( i=0, p[i]=strtok(vars," \n"); p[i]; p[++i]=strtok(NULL," \n") );
	for( i=0; p[i]; i=i+3 ) {
		ASCIItoOID( p[i], (n=mOiNew(64)) );
		MngrReadValue( p[i+1], p[i+2], (vb=mVbNew(64)), n );
		mVlPut( vbl, vb);
	}
	SNMP_DEBUG_API=4;
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrNetMonitor
 *
 * Arguments:	type, the arrived management communication type (OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function monitors the snmp layer.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrNetMonitor( SPTYPE *type )
{
	return( snmpLook( type ) );
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    testCommand
 *
 * Arguments:	test, single simple test (IN)
 *              foutput, reference to logfile (IN/OUT)
 *
 * Return:  	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:		Process single simple test and log response.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS 
testCommand( char *test, FILE *foutput , char *output_widget_name , char **test_string )
{
	char apiType[MaxBuf], targetHostName[MaxBuf], applicationName[MaxBuf], 
		 v1[MaxBuf], *v , testline[ MaxBuf ];
	int requestId, qos, maxRepetitions, nonRepeaters, i;
	VarBindList *vbl=mVlNew();
	SPTYPE type=0;

	strcpy( testline , test );
	fputs( testline, foutput );
	xRESPlaceStringinText( test_string , testline );
	xRESPrintStringGo2End( output_widget_name , *test_string );
	if( strncmp( testline, GETBULKREQUEST, strlen(GETBULKREQUEST) ) == 0 ) {
		sscanf( testline, "%s %s %s %d %d %d %d %s", 
			apiType, targetHostName, applicationName, &requestId, &qos, 
			&maxRepetitions, &nonRepeaters, v1 );
	} else {
		sscanf( testline, "%s %s %s %d %d %s" , apiType, targetHostName,
			 applicationName, &requestId, &qos, v1 );
	}
	if( strcmp( apiType, SETREQUEST ) == 0 ) {
		v = strstr( testline, v1 ); mapValues( v, vbl );
	} else {
		v = strstr( testline, v1 ); mapV( v, vbl );
	}

	if(	! strcmp( apiType, GETREQUEST) || 
		! strcmp( apiType, GETNEXTREQUEST ) ||
		! strcmp( apiType, GETBULKREQUEST ) ||
		! strcmp( apiType, SETREQUEST ) ||
		! strcmp( apiType, INFORMREQUEST ) ) 
	{
		if( strcmp( applicationName, "NULL" ) == 0 ) applicationName[0] = '\0';
		if(	! strcmp( apiType, GETREQUEST ) ) 
			snmpGetRequest( targetHostName, applicationName, requestId, qos, vbl );
		if(	! strcmp( apiType, GETNEXTREQUEST ) )  
			snmpGetNextRequest(targetHostName, applicationName, requestId, qos, vbl );
		if(	! strcmp( apiType, GETBULKREQUEST ) )  
			snmpGetBulkRequest( targetHostName, applicationName, requestId, qos, 
								maxRepetitions, nonRepeaters, vbl);
		if( ! strcmp( apiType, SETREQUEST ) ) 
			snmpSetRequest( targetHostName, applicationName, requestId, qos, vbl );
		if( ! strcmp( apiType , INFORMREQUEST ) ) 
			snmpInformRequest( targetHostName, applicationName, requestId, qos, vbl );
		while (MngrNetMonitor( &type ) != OK);
		return( MngrHandleNet( type, foutput , output_widget_name , test_string ) );
	}
	else 
		return FAILED;
	return OK ;
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    processScenario
 *
 * Arguments:	scenario, test scenario (IN)
 *				foutput, reference to logfile (IN/OUT)
 *
 * Return:  	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:		Process testscenario.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS 
processScenario( command *c, FILE *foutput , char *output_widget_name , char **test_string )
{
	if( !c ) return OK;
	switch( c->type ) {
 		case COMMENT: 
			fputs( c->line, foutput );
			xRESPlaceStringinText( test_string , c->line );
			xRESPrintStringGo2End( output_widget_name , *test_string );
			break;
		case COMMAND:
			if( testCommand( c->line, foutput , output_widget_name , test_string ) != OK ) return FAILED;
	}
		if( processScenario( c->next, foutput , output_widget_name , test_string ) != OK ) return FAILED ;
	return OK;
}

/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    processCommand
 *
 * Arguments:	scenario, test scenario (IN)
 *				foutput, reference to logfile (IN/OUT)
 *
 * Return:  	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:		Process current command in testscenario.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
processCommand( command *c, FILE *foutput , char *output_widget_name , char **test_string )
{
	switch( c->type ) {
 		case COMMENT: 
			fputs( c->line, foutput );
			xRESPlaceStringinText( test_string , c->line );
			xRESPrintStringGo2End( output_widget_name , *test_string );
			break;
		case COMMAND:
			if( testCommand( c->line, foutput , output_widget_name , test_string ) != OK ) return FAILED;
	}
	return OK;
}

/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    readCommandFile
 *
 * Arguments:	finput, reference to scenariofile (IN)
 *				table, reference to testTable (OUT)
 *
 * Return:  	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:		This function reads the scenario into an internal
 *				double-linked list data-structure.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS 
readScenario( FILE *finput, command **table )
{
	command *c, *p=NULL , *cf=NULL;
	char buf[MaxBuf];

	while( fgets( buf, MaxBuf, finput ) )
	{
		if( !(c = (command *) Asn1Alloc( sizeof(command) )) )
			return FAILED;
		if( !(*table) ) *table = c;
		c->prev=cf;
		c->next=NULL;
		cf=c;
		strcpy( c->line, buf ); memset( buf, ' ', MaxBuf );
		if( *c->line == '!' ) c->type = COMMENT; else c->type = COMMAND;
		if( p ) p->next = c; p = c;
	}
	fclose( finput );
	return OK ;
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    main
 *
 * Arguments:	argc, argument count (IN)
 *			 	argv, argument vector (IN)
 *				arge, argument environment (IN)
 *
 * Return:  	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:		* Open scenario file & log file
 *				* Open snmpapi
 *				* Process testscenario
 *				* Close snmpapi & files
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 *
	STATUS
testprocess( int argc, char** argv, char **arge )
{
	FILE *finput, *foutput;
	command *scenario=NULL;


	if (argc!=4){
		fprintf( stderr, "%% %s <scenariofile> <logfile> <name>\n", argv[0] );
		exit( FAILED );
	}
	if( (finput = fopen( argv[1], "r" )) == NULL ) {
		Error( "unable to scenariofile %s", argv[1] );
		exit( FAILED  );
	}
	if( (foutput = fopen( argv[2], "a" )) == NULL ) {
		Error( "unable to creat logfile %s", argv[2] );
		exit( FAILED  );
	}
	if( snmpOpen( "tester", arge ) != OK ) {
		Error( "couldn't open UT-SNMP" );
		exit( FAILED  );
	}
	sleep(10);
	if( readScenario( finput, &scenario ) != OK ) {
		Error( "unable to process scenariofile %s", argv[1] );
        exit( FAILED  );
	}
	if( processScenario( scenario, foutput ) != OK ) {
		Error( "unable to process scenario" );
		exit( FAILED );
	}
	if( snmpClose( GRACEFULL ) ) {
		Error( "couldn't close UT-SNMP" );
		exit( FAILED );
	}
	exit( OK );
}
*/

/* -----------------------------------------------------------------------
 */
