/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:	ResourceExtendSet.h
 *
 * Date:	May 1994
 *
 * Purpose:  	The header of an extension of the Athena Widget Set including
 *              communication between application and resource
 *
 * Author:	Peter Peters
 *
 * Remarks:	This is the newest non-conflicting extension file also for xinit
 *
 * -----------------------------------------------------------------------
 */

#ifndef _ResourceExtendSet_h_
#define _ResourceExtendSet_h_

/* -------------------------------------------- *
 * the include files for the extended functions *
 * -------------------------------------------- */
#include <Intrinsic.h>
#include <X11/Wc/WcCreate.h>
#include <stdio.h>
#include <string.h>

#include <ErrorWarningMessages.h>

/* ------------------------------------- *
 * Standard defines for the extended set *
 * ------------------------------------- */

/* defines the return status of the functions */

#define Ok_Status 	0
#define Fault_Status 	1

/* maximum string length of a string */

#define maximum_string_length 200


/*++++++++++++++++++++++++++++++++++++++++++++++
 * Variable : current_active_widget
 * Arguments: none
 * resource : none
 * purpose  : Used by the xRESPrintString & xRESPrintStringnPause for
 *            determine the current widget from where the callback-
 * 	      function is started
 * remarks  : use  xRESInitVariables to initialize this variable
 *           
 *++++++++++++++++++++++++++++++++++++++++++++++
*/

extern Widget current_active_widget;

/* ------------------------------ */
/* prototypes of the extended set */
/* ------------------------------ */

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESInitVariables
 * Argument:  Widget current_widget
 * resource: none
 * purpose : initialize the variable for the output
 * remarks : NECESSARY for every declared function, which use xRES-functions
 *	     Must be put in the first line after the variable declaring
 *++++++++++++++++++++++++++++++++++++++++++++++
*/

int xRESInitVariables( Widget );

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESGetnString
 * Argument: string_number(in) , head_string(in) , return_string(out)
 * resource: none
 * purpose : Gets the n-e substring out of the head-string
 * remarks : returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/

int xRESGetnString( int , char * , char * );

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESPrintString
 * Argument: output_widget(in) , output_string(in)
 * resource: none
 * purpose : prints an outputtext to the outputwidget with WcSetValue
 * remarks : use the string name of the output widget, only for AsciiSrc.
 *  	     returns the status.
 *++++++++++++++++++++++++++++++++++++++++++++++
*/

int xRESPrintString( char * , char * );

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESPrintStringnPause
 * Argument: output_widget(in) , output_string(in) , time_sec(in)
 * resource: none
 * purpose : Prints the output_string with xRESPrintString and pauses time_sec seconds
 * remarks : returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/

int xRESPrintStringnPause( char * , char * , int );

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESPrintStringGo2End
 * Argument: output_widget(in) , output_string(in)
 * resource: none
 * purpose : prints an outputtext to the outputwidget with WcSetValue
 * remarks : use the string name of the output widget, only for AsciiSrc.
 *  	     returns the status.
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
int xRESPrintStringGo2End( char * , char * );

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESPrintStringGo2EndnPause
 * Argument: output_widget(in) , output_string(in) , time_sec(in)
 * resource: none
 * purpose : Prints the output_string with xRESPrintString and pauses time_sec seconds and go to end of text
 * remarks : returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/
int xRESPrintStringnPauseGo2End( char * , char * , int );

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESCheckIntegerString
 * Argument: int_string(in)
 * resource: none
 * purpose : Check if the string contains only digits
 * remarks : returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/

int xRESCheckIntegerString( char * );

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESCheckIntegerInRange
 * Argument: number(in), low_range(in) , high_range(in)
 * resource: none
 * purpose : check if an integer is in the specified range
 * remarks : returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/

int xRESCheckIntegerInRange( int , int , int );

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESPlaceStringinText
 * Argument: total_string(in/out) , append_string(in)
 * resource: none
 * purpose : Place append_string at the end of total_string
 * remarks : returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/

int xRESPlaceStringinText( char ** , char * );

/*++++++++++++++++++++++++++++++++++++++++++++++
 * Function: xRESitoa
 * Argument: integer(in) , string_number(out)
 * resource: none
 * purpose : convert an integer to ascii string
 * remarks : string_number must be an array of char, returns the status
 *++++++++++++++++++++++++++++++++++++++++++++++
*/

int xRESitoa( int , char * );
#endif
